package estructuras;

import java.util.*;

import interfaz.*;

/**
 * Clase de objetos capaces de aplicar el algoritmo de Borvka para hallar un rbol
 * generador mnimo
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class AplicadorBoruvkaMinimo extends AdaptadoraAplicadorAlgoritmo {

    private AreaDibujoGrafo areaDibujo;
    private GestorPanelOpcionesBoruvka gestorPanel;
    
    private final int NUM_NODOS;
    private final int NUM_ARISTAS;
    private ArrayList<Nodo> nodos;
    private ArrayList<Arista> aristas;
    private int[] idsNodosOrigen;
    private int[] idsNodosDestino;
    private Vector<Integer> aristasOrdenadas;
    
    /*
     * Variables a actualizar en cada paso
     */
    private ArrayList<ArrayList<Integer>> aristasCandidatas;
    private int numComponentesPrincipioPaso;
    
    /*
     * Variables a actualizar en cada subpaso
     */
    private int componenteActual;
    private int[] componentes;
    //Indica, para cada arista, si se ha decidido aadirla al bosque en el paso actual
    private boolean[] aAnyadir;
    
    
    
    /**
     * Constructor
     * @param area rea de dibujo del grafo
     * @param panelOpciones panel de opciones disponibles durante la aplicacin de un algoritmo
     * @param barra barra de mens
     */
    public AplicadorBoruvkaMinimo(AreaDibujoGrafo area, PanelOpciones panelOpciones,
            BarraMenus barraMenus) {
        nodos = area.getNodos();
        NUM_NODOS = nodos.size();
        aristas = area.getAristas();
        NUM_ARISTAS = aristas.size();
        idsNodosOrigen = area.getIdsNodosOrigen();
        idsNodosDestino = area.getIdsNodosDestino();
        inicializarAristasOrdenadas();
        componenteActual = 0;
        componentes = new int[NUM_NODOS];
        for (int i = 0; i < NUM_NODOS; i++)
            componentes[i] = i;
        numComponentesPrincipioPaso = NUM_NODOS;
        aAnyadir = new boolean[NUM_ARISTAS];
        Arrays.fill(aAnyadir, false);
        actualizarAristasCandidatas();
        
        areaDibujo = area;
        gestorPanel = new GestorPanelOpcionesBoruvka(panelOpciones, area, barraMenus, this,
                NUM_NODOS, true);
        gestorPanel.actualizarPanel(listaComponentes());
        prepararAreaDibujo(gestorPanel.getNombresNodos());
    }
    
    /**
     * Determina si el paso de elegir arista es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @param idAristaElegida la arista elegida
     * @return true si el paso es correcto; false en caso contrario
     */
    public boolean elegirArista(int idArista) {
        if (numComponentesPrincipioPaso == 1)
            return false;
        
        ArrayList<Integer> candidatasComponente = aristasCandidatas.get(componenteActual);
        int numCandidatas = candidatasComponente.size();
        for (int i = 0; i < numCandidatas; i++) {
            
            //Si la arista candidata actual no es la elegida, se busca otra arista candidata
            if (candidatasComponente.get(i).intValue() != idArista)
                continue;
            
            int idOrigen = idsNodosOrigen[idArista];
            int idDestino = idsNodosDestino[idArista];
            int componenteOrigen = componentes[idOrigen];
            int componenteDestino = componentes[idDestino];
            if (componenteOrigen == componenteDestino) {
                if (!aAnyadir[idArista])
                    return false;
                actualizarComponentes(idOrigen, idDestino);
                areaDibujo.marcarArista(idArista);
                if ((componenteActual + 1) < numComponentesPrincipioPaso) {
                    componenteActual++;
                    gestorPanel.actualizarPanel();
                }
                else {
                    componenteActual = 0;
                    numComponentesPrincipioPaso = actualizarComponentes();
                    Arrays.fill(aAnyadir, false);
                    actualizarAristasCandidatas();
                    gestorPanel.actualizarPanel(listaComponentes());
                }
                return true;
            }
            else {
                aAnyadir[idArista] = true;
                actualizarComponentes(idOrigen, idDestino);
                areaDibujo.marcarArista(idArista);
                if ((componenteActual + 1) < numComponentesPrincipioPaso) {
                    componenteActual++;
                    gestorPanel.actualizarPanel();
                }
                else {
                    componenteActual = 0;
                    numComponentesPrincipioPaso = actualizarComponentes();
                    Arrays.fill(aAnyadir, false);
                    actualizarAristasCandidatas();
                    gestorPanel.actualizarPanel(listaComponentes());
                }
                return true;
            }
            
        }
        
        return false;
    }
    
    /**
     * Determina si el paso de terminar es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @return true si el paso es correcto; false en caso contrario
     */
    public boolean terminar() {
        if (numComponentesPrincipioPaso > 1)
            return false;
        gestorPanel.actualizarPanel(null);
        return true;
    }
    
    /**
     * Introduce las modificaciones oportunas en el area de dibujo teniendo en cuenta
     * que en dicho area se desea que quede reflejado el resultado de aplicar un paso
     * correcto del algoritmo.
     * Adems, actualiza el panel de opciones.
     */
    public void solucionPasoProximo() {
        if (numComponentesPrincipioPaso == 1) {
            gestorPanel.actualizarPanel(null);
            return;
        }
        
        ArrayList<Integer> candidatasComponente = aristasCandidatas.get(componenteActual);
        int numCandidatas = candidatasComponente.size();
        for (int i = 0; i < numCandidatas; i++) {
            
            int idArista = candidatasComponente.get(i).intValue();
            
            int idOrigen = idsNodosOrigen[idArista];
            int idDestino = idsNodosDestino[idArista];
            int componenteOrigen = componentes[idOrigen];
            int componenteDestino = componentes[idDestino];
            if (componenteOrigen == componenteDestino) {
                if (!aAnyadir[idArista])
                    continue;
                actualizarComponentes(idOrigen, idDestino);
                areaDibujo.marcarArista(idArista);
                if ((componenteActual + 1) < numComponentesPrincipioPaso) {
                    componenteActual++;
                    gestorPanel.actualizarPanel();
                }
                else {
                    componenteActual = 0;
                    numComponentesPrincipioPaso = actualizarComponentes();
                    Arrays.fill(aAnyadir, false);
                    actualizarAristasCandidatas();
                    gestorPanel.actualizarPanel(listaComponentes());
                }
                return;
            }
            else {
                aAnyadir[idArista] = true;
                actualizarComponentes(idOrigen, idDestino);
                areaDibujo.marcarArista(idArista);
                if ((componenteActual + 1) < numComponentesPrincipioPaso) {
                    componenteActual++;
                    gestorPanel.actualizarPanel();
                }
                else {
                    componenteActual = 0;
                    numComponentesPrincipioPaso = actualizarComponentes();
                    Arrays.fill(aAnyadir, false);
                    actualizarAristasCandidatas();
                    gestorPanel.actualizarPanel(listaComponentes());
                }
                return;
            }
            
        }
        
    }
    
    /**
     * Introduce las modificaciones oportunas en el area de dibujo teniendo en cuenta
     * que en dicho area se desea que quede reflejado el resultado de aplicar pasos
     * correctos del algoritmo hasta construir el rbol generador completo.
     * Adems, actualiza el panel de opciones.
     */
    public void solucionPasosRestantes() {
        while (numComponentesPrincipioPaso > 1) {
            ArrayList<Integer> candidatasComponente = aristasCandidatas.get(componenteActual);
            int numCandidatas = candidatasComponente.size();
            boolean encontradaArista = false;
            for (int i = 0; i < numCandidatas; i++) {
                
                int idArista = candidatasComponente.get(i).intValue();
                
                int idOrigen = idsNodosOrigen[idArista];
                int idDestino = idsNodosDestino[idArista];
                int componenteOrigen = componentes[idOrigen];
                int componenteDestino = componentes[idDestino];
                if (componenteOrigen == componenteDestino) {
                    if (!aAnyadir[idArista])
                        continue;
                    actualizarComponentes(idOrigen, idDestino);
                    areaDibujo.marcarArista(idArista);
                    if ((componenteActual + 1) < numComponentesPrincipioPaso) {
                        componenteActual++;
                        gestorPanel.actualizarPanel();
                    }
                    else {
                        componenteActual = 0;
                        numComponentesPrincipioPaso = actualizarComponentes();
                        Arrays.fill(aAnyadir, false);
                        actualizarAristasCandidatas();
                        gestorPanel.actualizarPanel(listaComponentes());
                    }
                    encontradaArista = true;
                    break;
                }
                else {
                    aAnyadir[idArista] = true;
                    actualizarComponentes(idOrigen, idDestino);
                    areaDibujo.marcarArista(idArista);
                    if ((componenteActual + 1) < numComponentesPrincipioPaso) {
                        componenteActual++;
                        gestorPanel.actualizarPanel();
                    }
                    else {
                        componenteActual = 0;
                        numComponentesPrincipioPaso = actualizarComponentes();
                        Arrays.fill(aAnyadir, false);
                        actualizarAristasCandidatas();
                        gestorPanel.actualizarPanel(listaComponentes());
                    }
                    encontradaArista = true;
                    break;
                }
                
            }
            
            if (!encontradaArista)
                return;
            
        }
        
        gestorPanel.actualizarPanel(null);
    }
    
    /**
     * Indica cul es la salida del algoritmo.
     * @return una cadena que indica cul es la salida del algoritmo.
     */
    public String salida() {
        return "minimum spanning tree";
    }
    
    /**
     * Inicializa la variable aristasOrdenadas (lista de identificadores de aristas ordenada
     * de menor a mayor peso de arista y, a igualdad de peso, de menor a mayor identificador)
     */
    private void inicializarAristasOrdenadas() {
        aristasOrdenadas = new Vector<Integer>();
        int numAristasOrdenadas = 0;
        for (int idAristaAInsertar = 0; idAristaAInsertar < NUM_ARISTAS; idAristaAInsertar++) {
            boolean insertado = false;
            int pesoAristaAInsertar = aristas.get(idAristaAInsertar).getPeso();
            for (int i = 0; i < numAristasOrdenadas; i++) {
                if (pesoAristaAInsertar <
                        aristas.get(aristasOrdenadas.elementAt(i).intValue()).getPeso()) {
                    aristasOrdenadas.insertElementAt(new Integer(idAristaAInsertar), i);
                    insertado = true;
                    break;
                }
            }
            if (!insertado)
                aristasOrdenadas.add(new Integer(idAristaAInsertar));
            numAristasOrdenadas++;
        }
    }
    
    /**
     * Inicializa la variable aristasCandidatas (lista de listas de identificadores de aristas
     * candidatas para cada componente conexa)
     */
    private void actualizarAristasCandidatas() {
        aristasCandidatas = new ArrayList<ArrayList<Integer>>();
        
        //Se crean tantas sublistas como componentes conexas
        for (int i = 0; i < numComponentesPrincipioPaso; i++)
            aristasCandidatas.add(new ArrayList<Integer>());
        
        //Si slo hay una componente, se deja la sublista correspondiente vaca
        if (numComponentesPrincipioPaso == 1)
            return;
        
        for (int i = 0; i < NUM_ARISTAS; i++) {
            int idArista = aristasOrdenadas.elementAt(i).intValue();
            int pesoArista = aristas.get(idArista).getPeso();
            
            //Si la arista no une dos componentes, no se aade a ninguna sublista
            int idOrigen = idsNodosOrigen[idArista];
            int idDestino = idsNodosDestino[idArista];
            int componenteOrigen = componentes[idOrigen];
            int componenteDestino = componentes[idDestino];
            if (componenteOrigen == componenteDestino)
                continue;
            
            //Si la sublista correspondiente a la componente del nodo origen est vaca o
            //si el peso de la arista es igual al de las aristas ya existentes en dicha
            //sublista, la arista se aade a la sublista
            ArrayList<Integer> aristasOrigen = aristasCandidatas.get(componenteOrigen);
            if (aristasOrigen.size() == 0)
                aristasOrigen.add(new Integer(idArista));
            else {
                if (pesoArista == aristas.get(aristasOrigen.get(0).intValue()).getPeso())
                    aristasOrigen.add(new Integer(idArista));
            }
            
            //Si la sublista correspondiente a la componente del nodo destino est vaca o
            //si el peso de la arista es igual al de las aristas ya existentes en dicha
            //sublista, la arista se aade a la sublista
            ArrayList<Integer> aristasDestino = aristasCandidatas.get(componenteDestino);
            if (aristasDestino.size() == 0)
                aristasDestino.add(new Integer(idArista));
            else {
                if (pesoArista == aristas.get(aristasDestino.get(0).intValue()).getPeso())
                    aristasDestino.add(new Integer(idArista));
            }
            
        }
    }
    
    /**
     * Prepara el rea de dibujo teniendo en cuenta que se va a empezar
     * a aplicar el algoritmo para hallar un rbol generador.
     * @param etiquetasNodos array de etiquetas de los nodos
     */
    private void prepararAreaDibujo(String[] etiquetasNodos) {
        areaDibujo.etiquetarNodos(etiquetasNodos);
        for (int i = 0; i < NUM_NODOS; i++)
            areaDibujo.marcarNodo(i);
    }
    
    /**
     * Devuelve una lista de listas de identificadores de nodos de las componentes conexas
     * @return la lista descrita anteriormente
     */
    private ArrayList<ArrayList<Integer>> listaComponentes() {
        ArrayList<ArrayList<Integer>> listaComponentes = new ArrayList<ArrayList<Integer>>();
        
        //Se crean tantas sublistas como componentes conexas
        for (int i = 0; i < numComponentesPrincipioPaso; i++)
            listaComponentes.add(new ArrayList<Integer>());
        
        //Si existe slo una componente, se aaden todos los nodos a la sublista
        //correspondiente a la componente y se devuelve la lista
        if (numComponentesPrincipioPaso == 1) {
            ArrayList<Integer> listaNodos = listaComponentes.get(0);
            for (int i = 0; i < NUM_NODOS; i++)
                listaNodos.add(new Integer(i));
            return listaComponentes;
        }
        
        //Se aade cada nodo a la sublista de la componente a la que pertenece
        for (int i = 0; i < NUM_NODOS; i++) {
            listaComponentes.get(componentes[i]).add(new Integer(i));
        }
        
        return listaComponentes;
    }
    
    /**
     * Actualiza el array que indica a qu componente conexa pertenece cada nodo del
     * grafo teniendo en cuenta que dos nodos pasan a pertenecer a la misma componente
     * conexa
     * @param idNodo1 identificador de uno de los dos nodos en cuestin
     * @param idNodo2 identificador del otro nodo
     */
    private void actualizarComponentes(int idNodo1, int idNodo2) {
        int compSustituta = componentes[idNodo1];
        int compASustituir = componentes[idNodo2];
        for (int i = 0; i < componentes.length; i++) {
            if (componentes[i] == compASustituir)
                componentes[i] = compSustituta;
        }
    }
    
    /**
     * Actualiza el array que indica a qu componente conexa pertenece cada nodo del grafo
     * teniendo en cuenta que se va a asignar a cada componente conexa un identificador
     * en el rango [0..(num_componentes - 1)], donde num_componentes es el nmero de
     * componentes conexas
     * @return el nmero de componentes conexas
     */
    private int actualizarComponentes() {
        int[] nuevoId = new int[NUM_NODOS];
        Arrays.fill(nuevoId, -1);
        int numNuevasComponentes = 0;
        for (int i = 0; i < NUM_NODOS; i++) {
            if (nuevoId[componentes[i]] < 0) {
                nuevoId[componentes[i]] = numNuevasComponentes;
                numNuevasComponentes++;
            }
            componentes[i] = nuevoId[componentes[i]];
        }
        return numNuevasComponentes;
    }

}
