package estructuras;

import interfaz.*;

import java.util.*;

/**
 * Clase de objetos capaces de aplicar un algoritmo para hallar un camino de
 * capacidad mxima entre dos vrtices de un grafo
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class AplicadorCaminoCapacidadMaxima extends AdaptadoraAplicadorAlgoritmo {

    private AreaDibujoGrafo areaDibujo;
    private GestorPanelOpcionesCaminoCapacidadMaxima gestorPanel;
    
    private final int NUM_NODOS;
    private final int NUM_ARISTAS;
    private ArrayList<Nodo> nodos;
    private ArrayList<Arista> aristas;
    private int[][] adyacencias;
    private int[] idsNodosOrigen;
    private int[] idsNodosDestino;
    private static final int MIN_VALUE = 0;
    
    //Nodos extremos del camino de capacidad mxima a encontrar
    private int idOrigenCamino = -1;
    private int idDestinoCamino = -1;
    
    //Indica, para cada nodo, la arista que se ha aadido al rbol para alcanzar el nodo
    //(si es que se ha aadido una arista para alcanzarlo)
    private int[] aristaANodo;
    
    //Indica, para cada arista del grafo, si la arista est en el camino de capacidad mxima
    private boolean[] enCamino;
    
    //Nmero de aristas en el camino de capacidad mxima
    private int numAristasEnCamino = 0;
    
    //Indica, para cada arista del grafo, si la arista se ha elegido para el camino de
    //capacidad mxima
    private boolean[] elegida;
    
    //Nmero de aristas elegidas para el camino de capacidad mxima
    private int numAristasElegidas = 0;
    
    /* Pasos del algoritmo:
     *  - ELEGIR_VERTICE
     *  - HALLAR_ARBOL
     *  - ELEGIR_ARISTA
     *  - TERMINAR
     */
    private final int ELEGIR_VERTICE = 0;
    private final int HALLAR_ARBOL = 1;
    private final int ELEGIR_ARISTA = 2;
    private final int TERMINAR = 3;
    
    //Indica el siguiente paso que hay que dar
    private int sigPaso = ELEGIR_VERTICE;
    
    
    /**
     * Constructor
     * @param area rea de dibujo del grafo
     * @param panelOpciones panel de opciones disponibles durante la aplicacin de un algoritmo
     * @param barra barra de mens
     */
    public AplicadorCaminoCapacidadMaxima(AreaDibujoGrafo area, PanelOpciones panelOpciones,
            BarraMenus barraMenus) {
        nodos = area.getNodos();
        NUM_NODOS = nodos.size();
        aristas = area.getAristas();
        NUM_ARISTAS = aristas.size();
        adyacencias = area.getAdyacencias();
        idsNodosOrigen = area.getIdsNodosOrigen();
        idsNodosDestino = area.getIdsNodosDestino();
        enCamino = new boolean[NUM_ARISTAS];
        Arrays.fill(enCamino, false);
        elegida = new boolean[NUM_ARISTAS];
        Arrays.fill(elegida, false);
        
        areaDibujo = area;
        gestorPanel = new GestorPanelOpcionesCaminoCapacidadMaxima(panelOpciones, area,
                barraMenus, this);
    }
    
    /**
     * Determina si el paso de elegir vrtice es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @param idVertice el vrtice elegido
     * @return true si el paso es correcto; false en caso contrario
     */
    public boolean elegirVertice(int idVertice) {
        if (sigPaso != ELEGIR_VERTICE)
            return false;
        if (idOrigenCamino < 0) {
            idOrigenCamino = idVertice;
            areaDibujo.marcarNodo(idVertice);
            return true;
        }
        if (idVertice == idOrigenCamino)
            return false;
        idDestinoCamino = idVertice;
        areaDibujo.marcarNodo(idVertice);
        sigPaso = HALLAR_ARBOL;
        return true;
    }
    
    /**
     * Determina si el paso de elegir arista es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @param idArista la arista elegida
     * @return true si el paso es correcto; false en caso contrario
     */
    public boolean elegirArista(int idArista) {
        if (sigPaso != ELEGIR_ARISTA)
            return false;
        if (!enCamino[idArista])
            return false;
        if (elegida[idArista])
            return false;
        elegida[idArista] = true;
        numAristasElegidas++;
        areaDibujo.marcarNodo(idsNodosOrigen[idArista]);
        areaDibujo.marcarNodo(idsNodosDestino[idArista]);
        areaDibujo.marcarArista(idArista);
        if (numAristasElegidas == numAristasEnCamino)
            sigPaso = TERMINAR;
        return true;
    }
    
    /**
     * Determina si el paso de hallar rbol generador mximo es correcto y,
     * en caso positivo, introduce las modificaciones oportunas en el area de dibujo y
     * actualiza el panel de opciones
     * @return true si el paso es correcto; false en caso contrario
     */
    public boolean hallarArbolGeneradorMaximo() {
        if (sigPaso != HALLAR_ARBOL)
            return false;
        aristaANodo = pintarArbol();
        
        //Se averigua cuntas aristas estn en el camino de capacidad mxima y cules son
        int idNodoActual = idDestinoCamino;
        do {
            int idArista = aristaANodo[idNodoActual];
            enCamino[idArista] = true;
            numAristasEnCamino++;
            if (idsNodosOrigen[idArista] != idNodoActual)
                idNodoActual = idsNodosOrigen[idArista];
            else
                idNodoActual = idsNodosDestino[idArista];
        } while (idNodoActual != idOrigenCamino);
        
        sigPaso = ELEGIR_ARISTA;
        return true;
    }
    
    /**
     * Determina si el paso de terminar es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @return true si el paso es correcto; false en caso contrario
     */
    public boolean terminar() {
        if (sigPaso != TERMINAR)
            return false;
        gestorPanel.actualizarPanel();
        sigPaso = -1;
        return true;
    }
    
    /**
     * Introduce las modificaciones oportunas en el area de dibujo teniendo en cuenta
     * que en dicho area se desea que quede reflejado el resultado de aplicar un paso
     * correcto del algoritmo.
     * Adems, actualiza el panel de opciones.
     */
    public void solucionPasoProximo() {
        int idNodoActual;
        switch (sigPaso) {
        case ELEGIR_VERTICE:
            if (idOrigenCamino < 0) {
                idOrigenCamino = 0;
                areaDibujo.marcarNodo(0);
                return;
            }
            idDestinoCamino = 1;
            areaDibujo.marcarNodo(1);
            sigPaso = HALLAR_ARBOL;
            break;
        case HALLAR_ARBOL:
            aristaANodo = pintarArbol();
            
            //Se averigua cuntas aristas estn en el camino de capacidad mxima y cules son
            idNodoActual = idDestinoCamino;
            do {
                int idArista = aristaANodo[idNodoActual];
                enCamino[idArista] = true;
                numAristasEnCamino++;
                if (idsNodosOrigen[idArista] != idNodoActual)
                    idNodoActual = idsNodosOrigen[idArista];
                else
                    idNodoActual = idsNodosDestino[idArista];
            } while (idNodoActual != idOrigenCamino);
            
            sigPaso = ELEGIR_ARISTA;
            break;
        case ELEGIR_ARISTA:
            idNodoActual = idDestinoCamino;
            do {
                int idArista = aristaANodo[idNodoActual];
                if (!elegida[idArista]) {
                    elegida[idArista] = true;
                    numAristasElegidas++;
                    areaDibujo.marcarNodo(idsNodosOrigen[idArista]);
                    areaDibujo.marcarNodo(idsNodosDestino[idArista]);
                    areaDibujo.marcarArista(idArista);
                    if (numAristasElegidas == numAristasEnCamino)
                        sigPaso = TERMINAR;
                    return;
                }
                if (idsNodosOrigen[idArista] != idNodoActual)
                    idNodoActual = idsNodosOrigen[idArista];
                else
                    idNodoActual = idsNodosDestino[idArista];
            } while (idNodoActual != idOrigenCamino);
            break;
        case TERMINAR:
            gestorPanel.actualizarPanel();
            sigPaso = -1;
            break;
        }
    }
    
    /**
     * Introduce las modificaciones oportunas en el area de dibujo teniendo en cuenta
     * que en dicho area se desea que quede reflejado el resultado de aplicar pasos
     * correctos del algoritmo hasta construir el rbol generador completo.
     * Adems, actualiza el panel de opciones.
     */
    public void solucionPasosRestantes() {
        while (sigPaso != -1)
            solucionPasoProximo();
    }
    
    /**
     * Indica cul es la salida del algoritmo.
     * @return una cadena que indica cul es la salida del algoritmo.
     */
    public String salida() {
        return "maximum capacity path";
    }
    
    /**
     * Pinta un rbol generador mximo del grafo
     * @return un array que indica, para cada nodo, la arista que se ha aadido al rbol
     * para alcanzar el nodo (si es que se ha aadido una arista para alcanzarlo)
     */
    private int[] pintarArbol() {
        int[] aristaANodo = new int[NUM_NODOS];
        Arrays.fill(aristaANodo, -1);
        
        ArrayList<Integer> nodosEnArbol = new ArrayList<Integer>();
        ArrayList<Integer> nodosNoEnArbol = new ArrayList<Integer>();
        ArrayList<Integer> etiquetasNodosNoEnArbol = new ArrayList<Integer>();
        for (int i = 0; i < NUM_NODOS; i++) {
            nodosNoEnArbol.add(new Integer(i));
            etiquetasNodosNoEnArbol.add(new Integer(MIN_VALUE));
        }
        int etiquetaMaxima = MIN_VALUE;
        
        //Se aade el primer nodo al rbol
        int idNodoAAnyadir = nodosNoEnArbol.get(idOrigenCamino).intValue();
        etiquetaMaxima = anyadirNodoAArbol(idOrigenCamino, nodosEnArbol, nodosNoEnArbol,
                etiquetasNodosNoEnArbol);
        areaDibujo.marcarNodoArbolAux(idNodoAAnyadir);
        
        while (nodosEnArbol.size() < NUM_NODOS) {
            
            //Se busca un nodo no en rbol con etiqueta mxima
            int numNodosNoEnArbol = nodosNoEnArbol.size();
            int iNodoAAnyadir = -1;
            for (int i = 0; i < numNodosNoEnArbol; i++) {
                int etiquetaActual = etiquetasNodosNoEnArbol.get(i).intValue();
                if (etiquetaActual == etiquetaMaxima) {
                    iNodoAAnyadir = i;
                    break;
                }
            }
            idNodoAAnyadir = nodosNoEnArbol.get(iNodoAAnyadir).intValue();
            
            //Se busca una arista cuyo peso coincida con la etiqueta mxima que una el
            //nodo encontrado con un nodo del rbol
            int idAristaAAnyadir = -1;
            int numNodosEnArbol = nodosEnArbol.size();
            for (int i = 0; i < numNodosEnArbol; i++) {
                int idNodoEnArbol = nodosEnArbol.get(i).intValue();
                int idArista = adyacencias[idNodoAAnyadir][idNodoEnArbol];
                if (idArista >= 0) {
                    if (aristas.get(idArista).getPeso() == etiquetaMaxima) {
                        idAristaAAnyadir = idArista;
                        break;
                    }
                }
            }
            
            etiquetaMaxima = anyadirNodoAArbol(iNodoAAnyadir, nodosEnArbol, nodosNoEnArbol,
                    etiquetasNodosNoEnArbol);
            areaDibujo.marcarNodoArbolAux(idNodoAAnyadir);
            areaDibujo.marcarAristaArbolAux(idAristaAAnyadir);
            aristaANodo[idNodoAAnyadir] = idAristaAAnyadir;
        }
        
        return aristaANodo;
    }
    
    /**
     * Pasa un nodo de una lista de nodos no en rbol a una lista de nodos en rbol,
     * actualiza una lista de etiquetas de los nodos no en rbol y calcula
     * el valor de la etiqueta mxima
     * @param i indice para acceder al nodo dentro de la lista de nodos no en rbol
     * @param nodosEnArbol lista de nodos no en rbol
     * @param nodosNoEnArbol lista de nodos no en rbol
     * @param etiquetasNodosNoEnArbol lista de etiquetas de los nodos no en rbol
     * @return el valor de la etiqueta mxima
     */
    private int anyadirNodoAArbol(int i, ArrayList<Integer> nodosEnArbol,
            ArrayList<Integer> nodosNoEnArbol, ArrayList<Integer> etiquetasNodosNoEnArbol) {
        int idNodo = nodosNoEnArbol.get(i).intValue();
        nodosEnArbol.add(nodosNoEnArbol.remove(i));
        etiquetasNodosNoEnArbol.remove(i);
        int numNodosNoEnArbol = nodosNoEnArbol.size();
        int etiquetaMaxima = MIN_VALUE;
        for (int iNodoNoEnArbol = 0; iNodoNoEnArbol < numNodosNoEnArbol; iNodoNoEnArbol++) {
            int idArista = adyacencias[idNodo][nodosNoEnArbol.get(iNodoNoEnArbol).intValue()];
            if (idArista >= 0) {
                int nuevaEtiqueta = aristas.get(idArista).getPeso();
                if (nuevaEtiqueta > etiquetasNodosNoEnArbol.get(iNodoNoEnArbol).intValue())
                    etiquetasNodosNoEnArbol.set(iNodoNoEnArbol, new Integer(nuevaEtiqueta));
            }
            int etiquetaActual = etiquetasNodosNoEnArbol.get(iNodoNoEnArbol).intValue();
            if (etiquetaActual > etiquetaMaxima)
                etiquetaMaxima = etiquetaActual;
        }
        return etiquetaMaxima;
    }
    
}
