package estructuras;

import java.util.*;

import interfaz.*;

/**
 * Clase de objetos capaces de aplicar el algoritmo de Galil y Schieber para hallar el rbol
 * generador ms uniforme
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class AplicadorGalilSchieberMasUniforme extends AdaptadoraAplicadorAlgoritmo {

    private AreaDibujoGrafo areaDibujo;
    private GestorPanelOpcionesGalilSchieber gestorPanel;
    
    private final int NUM_NODOS;
    private final int NUM_ARISTAS;
    private ArrayList<estructuras.Nodo> nodos;
    private ArrayList<Arista> aristas;
    private int[][] adyacencias;
    private int[] idsNodosOrigen;
    private int[] idsNodosDestino;
    private Vector<Integer> aristasOrdenadas;
    private Bosque bosqueOriginal;
    private Bosque bosqueCopia;  //para poder "deshacer cambios" en el bosque original
    
    //Indica qu arista se est escaneando
    private int idAristaActual;
    
    //Indica cul es la siguiente arista a escanear (ndice para acceder a aristasOrdenadas)
    private int iAristaSig = 0;
    
    //Mxima diferencia de pesos entre dos aristas del rbol ms uniforme
    //encontrado hasta el momento
    private int z = Integer.MAX_VALUE;
    
    //Aristas del rbol generador ms uniforme encontrado hasta el momento
    private ArrayList<Integer> t = new ArrayList<Integer>();
    
    //Aristas del bosque generador
    private ArrayList<Integer> s = new ArrayList<Integer>();
    
    //Indica, para cada arista, si est en el bosque generador
    private boolean[] enBosque;
    
    /* Estados en los que se puede encuentrar el aplicador:
     *  - INICIAL (no se ha dado ningn paso del algoritmo)
     *  - BORRADA_ARISTA (se acaba de borrar una arista del bosque generador)
     *  - ANYADIDA_ARISTA (se acaba de aadir una arista al bosque)
     *  - ELEGIDO_ARBOL (se acaba de elegir un rbol como el ms uniforme de los hallados
     *    hasta el momento)
     *  - FINAL (se han dado todos los pasos del algoritmo)
     */
    private final int INICIAL = 0;
    private final int BORRADA_ARISTA = 1;
    private final int ANYADIDA_ARISTA = 2;
    private final int ELEGIDO_ARBOL = 3;
    private final int FINAL = 4;
    
    //Indica el estado en el que se encuentra el aplicador
    private int estado = INICIAL;
    
    /**
     * Constructor
     * @param area rea de dibujo del grafo
     */
    public AplicadorGalilSchieberMasUniforme(AreaDibujoGrafo area, PanelOpciones panelOpciones,
            BarraMenus barraMenus) {
        nodos = area.getNodos();
        NUM_NODOS = nodos.size();
        aristas = area.getAristas();
        NUM_ARISTAS = aristas.size();
        adyacencias = area.getAdyacencias();
        idsNodosOrigen = area.getIdsNodosOrigen();
        idsNodosDestino = area.getIdsNodosDestino();
        inicializarAristasOrdenadas();
        bosqueOriginal = new Bosque();
        bosqueCopia = new Bosque();
        enBosque = new boolean[NUM_ARISTAS];
        Arrays.fill(enBosque, false);
        
        areaDibujo = area;
        gestorPanel = new GestorPanelOpcionesGalilSchieber(panelOpciones, area, barraMenus, this,
                aristasOrdenadas, true);
        prepararAreaDibujo(gestorPanel.getNombresAristas());
    }
    
    /**
     * Determina si el paso de elegir arista es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @param idArista la arista elegida
     * @return true si el paso es correcto; false en caso contrario
     */
    public boolean elegirArista(int idArista) {
        int idOrigen = idsNodosOrigen[idArista];
        int idDestino = idsNodosDestino[idArista];
        int idOrigenActual;
        int idDestinoActual;
        ArrayList<Integer> aristasCandidatas;
        if (enBosque[idArista]) {  //el usuario decide borrar la arista del bosque
            switch (estado) {
            case INICIAL:
            case BORRADA_ARISTA:
                return false;
            case ANYADIDA_ARISTA:
                if (s.size() == (NUM_NODOS - 1)) {
                    int idAristaPesoMin = s.get(0).intValue();
                    int idAristaPesoMax = s.get(NUM_NODOS - 2).intValue();
                    int nuevaDifMax = aristas.get(idAristaPesoMax).getPeso() -
                            aristas.get(idAristaPesoMin).getPeso();
                    if (nuevaDifMax < z)
                        return false;
                }
                if (iAristaSig == NUM_ARISTAS)
                    return false;
                idAristaActual = aristasOrdenadas.elementAt(iAristaSig).intValue();
                idOrigenActual = idsNodosOrigen[idAristaActual];
                idDestinoActual = idsNodosDestino[idAristaActual];
                if (bosqueOriginal.findroot(idOrigenActual) !=
                    bosqueOriginal.findroot(idDestinoActual))
                    return false;
                bosqueOriginal.evert(idOrigenActual);
                aristasCandidatas = bosqueOriginal.findcost(idDestinoActual);
                if (!estaEn(aristasCandidatas, idArista)) {
                    bosqueOriginal = bosqueCopia.copia();
                    return false;
                }
                bosqueCopia.evert(idOrigenActual);
                bosqueOriginal.cut(idOrigen, idDestino);
                bosqueCopia.cut(idOrigen, idDestino);
                iAristaSig++;
                borrar(s, idArista);
                enBosque[idArista] = false;
                estado = BORRADA_ARISTA;
                gestorPanel.actualizarPanel();
                areaDibujo.desmarcarArista(idArista);
                return true;
            case ELEGIDO_ARBOL:
                if (iAristaSig == NUM_ARISTAS)
                    return false;
                idAristaActual = aristasOrdenadas.elementAt(iAristaSig).intValue();
                idOrigenActual = idsNodosOrigen[idAristaActual];
                idDestinoActual = idsNodosDestino[idAristaActual];
                if (bosqueOriginal.findroot(idOrigenActual) !=
                    bosqueOriginal.findroot(idDestinoActual))
                    return false;
                bosqueOriginal.evert(idOrigenActual);
                aristasCandidatas = bosqueOriginal.findcost(idDestinoActual);
                if (!estaEn(aristasCandidatas, idArista)) {
                    bosqueOriginal = bosqueCopia.copia();
                    return false;
                }
                bosqueCopia.evert(idOrigenActual);
                bosqueOriginal.cut(idOrigen, idDestino);
                bosqueCopia.cut(idOrigen, idDestino);
                iAristaSig++;
                borrar(s, idArista);
                enBosque[idArista] = false;
                estado = BORRADA_ARISTA;
                gestorPanel.actualizarPanel();
                areaDibujo.desmarcarArista(idArista);
                return true;
            }
        }
        else {  //el usuario decide aadir la arista al bosque
            switch (estado) {
            case ELEGIDO_ARBOL:
                return false;
            case INICIAL:
                if (iAristaSig == NUM_ARISTAS)
                    return false;
                idAristaActual = aristasOrdenadas.elementAt(iAristaSig).intValue();
                if (idAristaActual != idArista)
                    return false;
                bosqueOriginal.link(idOrigen, idDestino);
                bosqueCopia.link(idOrigen, idDestino);
                iAristaSig++;
                s.add(new Integer(idArista));
                enBosque[idArista] = true;
                estado = ANYADIDA_ARISTA;
                gestorPanel.actualizarPanel();
                areaDibujo.marcarArista(idArista);
                return true;
            case BORRADA_ARISTA:
                if (idArista != idAristaActual)
                    return false;
                bosqueOriginal.link(idOrigen, idDestino);
                bosqueCopia.link(idOrigen, idDestino);
                s.add(new Integer(idArista));
                enBosque[idArista] = true;
                estado = ANYADIDA_ARISTA;
                areaDibujo.marcarArista(idArista);
                return true;
            case ANYADIDA_ARISTA:
                if (s.size() == (NUM_NODOS - 1)) {
                    int idAristaPesoMin = s.get(0).intValue();
                    int idAristaPesoMax = s.get(NUM_NODOS - 2).intValue();
                    int nuevaDifMax = aristas.get(idAristaPesoMax).getPeso() -
                            aristas.get(idAristaPesoMin).getPeso();
                    if (nuevaDifMax < z)
                        return false;
                }
                if (iAristaSig == NUM_ARISTAS)
                    return false;
                idAristaActual = aristasOrdenadas.elementAt(iAristaSig).intValue();
                idOrigenActual = idsNodosOrigen[idAristaActual];
                idDestinoActual = idsNodosDestino[idAristaActual];
                if (bosqueOriginal.findroot(idOrigenActual) ==
                    bosqueOriginal.findroot(idDestinoActual))
                    return false;
                if (idArista != idAristaActual)
                    return false;
                bosqueOriginal.link(idOrigen, idDestino);
                bosqueCopia.link(idOrigen, idDestino);
                iAristaSig++;
                s.add(new Integer(idArista));
                enBosque[idArista] = true;
                estado = ANYADIDA_ARISTA;
                gestorPanel.actualizarPanel();
                areaDibujo.marcarArista(idArista);
                return true;
            }
        }
        return false;
    }
    
    /**
     * Determina si el paso de elegir rbol es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @return true si el paso es correcto; false en caso contrario
     */
    public boolean elegirArbol() {
        switch (estado) {
        case INICIAL:
        case BORRADA_ARISTA:
        case ELEGIDO_ARBOL:
            return false;
        case ANYADIDA_ARISTA:
            if (s.size() != (NUM_NODOS - 1))
                return false;
            int idAristaPesoMin = s.get(0).intValue();
            int idAristaPesoMax = s.get(NUM_NODOS - 2).intValue();
            int nuevaDifMax = aristas.get(idAristaPesoMax).getPeso() -
                    aristas.get(idAristaPesoMin).getPeso();
            if (nuevaDifMax >= z)
                return false;
            z = nuevaDifMax;
            t = copiar(s);
            estado = ELEGIDO_ARBOL;
            gestorPanel.actualizarPanel(nuevaDifMax);
            return true;
        }
        return false;
    }
    
    /**
     * Determina si el paso de terminar es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @return true si el paso es correcto; false en caso contrario
     */
    public boolean terminar() {
        switch (estado) {
        case BORRADA_ARISTA:
            return false;
        case INICIAL:
        case ELEGIDO_ARBOL:
            if (iAristaSig < NUM_ARISTAS)
                return false;
            gestorPanel.actualizarPanel(-1);
            estado = FINAL;
            return true;
        case ANYADIDA_ARISTA:
            if (iAristaSig < NUM_ARISTAS)
                return false;
            int idAristaPesoMin = s.get(0).intValue();
            int idAristaPesoMax = s.get(NUM_NODOS - 2).intValue();
            int nuevaDifMax = aristas.get(idAristaPesoMax).getPeso() -
                    aristas.get(idAristaPesoMin).getPeso();
            if (nuevaDifMax < z)
                return false;
            gestorPanel.actualizarPanel(-1);
            areaDibujo.limpiarGrafo();
            prepararAreaDibujo(gestorPanel.getNombresAristas());
            int numAristas = t.size();
            for (int i = 0; i < numAristas; i++) {
                areaDibujo.marcarArista(t.get(i).intValue());
            }
            estado = FINAL;
            return true;
        }
        return false;
    }
    
    /**
     * Introduce las modificaciones oportunas en el area de dibujo teniendo en cuenta
     * que en dicho area se desea que quede reflejado el resultado de aplicar un paso
     * correcto del algoritmo.
     * Adems, actualiza el panel de opciones.
     */
    public void solucionPasoProximo() {
        int idOrigen;
        int idDestino;
        switch (estado) {
        case INICIAL:
            if (iAristaSig == NUM_ARISTAS) {
                gestorPanel.actualizarPanel(-1);
                estado = FINAL;
                return;
            }
            idAristaActual = aristasOrdenadas.elementAt(iAristaSig).intValue();
            idOrigen = idsNodosOrigen[idAristaActual];
            idDestino = idsNodosDestino[idAristaActual];
            
            //No hay que comprobar si los nodos extremos de la arista estn en el
            //mismo rbol (todos los nodos estn en distintos rboles)
            
            bosqueOriginal.link(idOrigen, idDestino);
            bosqueCopia.link(idOrigen, idDestino);
            iAristaSig++;
            s.add(new Integer(idAristaActual));
            enBosque[idAristaActual] = true;
            estado = ANYADIDA_ARISTA;
            gestorPanel.actualizarPanel();
            areaDibujo.marcarArista(idAristaActual);
            return;
        case BORRADA_ARISTA:
            idOrigen = idsNodosOrigen[idAristaActual];
            idDestino = idsNodosDestino[idAristaActual];
            bosqueOriginal.link(idOrigen, idDestino);
            bosqueCopia.link(idOrigen, idDestino);
            s.add(new Integer(idAristaActual));
            enBosque[idAristaActual] = true;
            estado = ANYADIDA_ARISTA;
            areaDibujo.marcarArista(idAristaActual);
            return;
        case ANYADIDA_ARISTA:
            if (s.size() == (NUM_NODOS - 1)) {
                int idAristaPesoMin = s.get(0).intValue();
                int idAristaPesoMax = s.get(NUM_NODOS - 2).intValue();
                int nuevaDifMax = aristas.get(idAristaPesoMax).getPeso() -
                        aristas.get(idAristaPesoMin).getPeso();
                if (nuevaDifMax < z) {
                    z = nuevaDifMax;
                    t = copiar(s);
                    estado = ELEGIDO_ARBOL;
                    gestorPanel.actualizarPanel(nuevaDifMax);
                    return;
                }
            }
            if (iAristaSig == NUM_ARISTAS) {
                gestorPanel.actualizarPanel(-1);
                areaDibujo.limpiarGrafo();
                prepararAreaDibujo(gestorPanel.getNombresAristas());
                int numAristas = t.size();
                for (int i = 0; i < numAristas; i++)
                    areaDibujo.marcarArista(t.get(i).intValue());
                estado = FINAL;
                return;
            }
            idAristaActual = aristasOrdenadas.elementAt(iAristaSig).intValue();
            idOrigen = idsNodosOrigen[idAristaActual];
            idDestino = idsNodosDestino[idAristaActual];
            if (bosqueOriginal.findroot(idOrigen) == bosqueOriginal.findroot(idDestino)) {
                bosqueOriginal.evert(idOrigen);
                bosqueCopia.evert(idOrigen);
                int idArista = bosqueOriginal.findcost(idDestino).get(0).intValue();
                idOrigen = idsNodosOrigen[idArista];
                idDestino = idsNodosDestino[idArista];
                bosqueOriginal.cut(idOrigen, idDestino);
                bosqueCopia.cut(idOrigen, idDestino);
                iAristaSig++;
                borrar(s, idArista);
                enBosque[idArista] = false;
                estado = BORRADA_ARISTA;
                gestorPanel.actualizarPanel();
                areaDibujo.desmarcarArista(idArista);
            }
            else {  //bosqueOriginal.findroot(idOrigen) != bosqueOriginal.findroot(idDestino)
                bosqueOriginal.link(idOrigen, idDestino);
                bosqueCopia.link(idOrigen, idDestino);
                iAristaSig++;
                s.add(new Integer(idAristaActual));
                enBosque[idAristaActual] = true;
                estado = ANYADIDA_ARISTA;
                gestorPanel.actualizarPanel();
                areaDibujo.marcarArista(idAristaActual);
            }
            return;
        case ELEGIDO_ARBOL:
            if (iAristaSig == NUM_ARISTAS) {
                gestorPanel.actualizarPanel(-1);
                estado = FINAL;
                return;
            }
            idAristaActual = aristasOrdenadas.elementAt(iAristaSig).intValue();
            idOrigen = idsNodosOrigen[idAristaActual];
            idDestino = idsNodosDestino[idAristaActual];
            
            //No hay que comprobar si los nodos extremos de la arista estn en el
            //mismo rbol (todos los nodos estn en el mismo rbol)
            
            bosqueOriginal.evert(idOrigen);
            bosqueCopia.evert(idOrigen);
            int idArista = bosqueOriginal.findcost(idDestino).get(0).intValue();
            idOrigen = idsNodosOrigen[idArista];
            idDestino = idsNodosDestino[idArista];
            bosqueOriginal.cut(idOrigen, idDestino);
            bosqueCopia.cut(idOrigen, idDestino);
            iAristaSig++;
            borrar(s, idArista);
            enBosque[idArista] = false;
            estado = BORRADA_ARISTA;
            gestorPanel.actualizarPanel();
            areaDibujo.desmarcarArista(idArista);
            return;
        }
    }
    
    /**
     * Introduce las modificaciones oportunas en el area de dibujo teniendo en cuenta
     * que en dicho area se desea que quede reflejado el resultado de aplicar pasos
     * correctos del algoritmo hasta construir el rbol generador completo.
     * Adems, actualiza el panel de opciones.
     */
    public void solucionPasosRestantes() {
        do {
            solucionPasoProximo();
        } while (estado != FINAL);
    }
    
    /**
     * Indica cul es la salida del algoritmo.
     * @return una cadena que indica cul es la salida del algoritmo.
     */
    public String salida() {
        return "most uniform spanning tree";
    }
    
    /**
     * Inicializa la variable aristasOrdenadas (lista de identificadores de aristas ordenada
     * de menor a mayor peso de arista y, a igualdad de peso, de menor a mayor identificador)
     */
    private void inicializarAristasOrdenadas() {
        aristasOrdenadas = new Vector<Integer>();
        int numAristasOrdenadas = 0;
        for (int idAristaAInsertar = 0; idAristaAInsertar < NUM_ARISTAS; idAristaAInsertar++) {
            boolean insertado = false;
            int pesoAristaAInsertar = aristas.get(idAristaAInsertar).getPeso();
            for (int i = 0; i < numAristasOrdenadas; i++) {
                if (pesoAristaAInsertar <
                        aristas.get(aristasOrdenadas.elementAt(i).intValue()).getPeso()) {
                    aristasOrdenadas.insertElementAt(new Integer(idAristaAInsertar), i);
                    insertado = true;
                    break;
                }
            }
            if (!insertado)
                aristasOrdenadas.add(new Integer(idAristaAInsertar));
            numAristasOrdenadas++;
        }
    }
    
    /**
     * Prepara el rea de dibujo teniendo en cuenta que se va a empezar
     * a aplicar el algoritmo para hallar un rbol generador.
     * @param etiquetasNodos array de etiquetas de los nodos
     */
    private void prepararAreaDibujo(String[] etiquetasAristas) {
        if (NUM_ARISTAS > 0)
            areaDibujo.etiquetarAristas(etiquetasAristas);
        for (int i = 0; i < NUM_NODOS; i++)
            areaDibujo.marcarNodo(i);
    }
    
    /**
     * Determina si una arista est en una lista de aristas
     * @param aristas la lista de aristas
     * @param idArista la arista
     * @return true en caso positivo; false en caso contrario
     */
    private boolean estaEn(ArrayList<Integer> aristas, int idArista) {
        int numAristas = aristas.size();
        for (int i = 0; i < numAristas; i++)
            if (aristas.get(i).intValue() == idArista)
                return true;
        return false;
    }
    
    /**
     * Borra una arista de una lista de aristas
     * @param aristas la lista de aristas
     * @param idArista la arista
     */
    private void borrar(ArrayList<Integer> aristas, int idArista) {
        int numAristas = aristas.size();
        for (int i = 0; i < numAristas; i++)
            if (aristas.get(i).intValue() == idArista) {
                aristas.remove(i);
                return;
            }
    }
    
    /**
     * Copia una lista de aristas
     * @param aristas la lista de aristas a copiar
     * @return la copia construida
     */
    private ArrayList<Integer> copiar(ArrayList<Integer> aristasOriginal) {
        ArrayList<Integer> copia = new ArrayList<Integer>();
        int numAristas = aristasOriginal.size();
        for (int i = 0; i < numAristas; i++)
            copia.add(new Integer(aristasOriginal.get(i).intValue()));
        return copia;
    }
    
    /**
     * Clase que representa el bosque generador que se mantendr durante la aplicacin del
     * algoritmo
     */
    class Bosque {
        
        private ArrayList<Nodo> nodos;
        
        /**
         * Constructor
         * 
         */
        public Bosque() {
            nodos = new ArrayList<Nodo>();
            for (int i = 0; i < NUM_NODOS; i++)
                nodos.add(new Nodo(i));
        }
        
        /**
         * Constructor
         * @param nodos nodos que contendr el nuevo bosque
         */
        public Bosque(ArrayList<Nodo> nodos) {
            this.nodos = nodos;
        }
        
        /**
         * Devuelve la raz del rbol que contiene el nodo recibido como parmetro
         * @param v un nodo
         * @return la raz del rbol que contiene el nodo recibido como parmetro
         */
        public int findroot(int v) {
            return nodos.get(v).getRaiz().getId();
        }
        
        /**
         * Devuelve las aristas de peso mnimo que se encuentren en el camino del nodo
         * recibido como parmetro a su raz
         * @param v un nodo
         * @return las aristas de peso mnimo que se encuentren en el camino del nodo
         * recibido como parmetro a su raz
         */
        public ArrayList<Integer> findcost(int v) {
            ArrayList<Integer> aristasPesoMinimo = new ArrayList<Integer>();
            Nodo hijo = nodos.get(v);
            Nodo padre = hijo.getPadre();
            int pesoMinimo = Integer.MAX_VALUE;
            while (padre != null) {
                int idArista = adyacencias[hijo.getId()][padre.getId()];
                int pesoArista = aristas.get(idArista).getPeso();
                if (pesoArista <= pesoMinimo) {
                    if (pesoArista < pesoMinimo) {
                        aristasPesoMinimo.clear();
                        pesoMinimo = pesoArista;
                    }
                    aristasPesoMinimo.add(new Integer(idArista));
                }
                hijo = padre;
                padre = hijo.getPadre();
            }
            return aristasPesoMinimo;
        }
        
        /**
         * Convierte el nodo recibido como parmetro en la raz del rbol en el que se
         * encuentra
         * @param v un nodo
         */
        public void evert(int v) {
            nodos.get(v).convertirEnRaiz();
        }
        
        /**
         * Combina los rboles que contienen los nodos "v" y "w" aadiendo la arista (v, w) y
         * haciendo a "v" la raz del nuevo rbol (se asume que "v" y "w" estn en rboles
         * distintos)
         * @param v un nodo
         * @param w otro nodo
         */
        public void link(int v, int w) {
            Nodo nodoV = nodos.get(v);
            Nodo nodoW = nodos.get(w);
            nodoV.convertirEnRaiz();
            nodoW.convertirEnRaiz();
            nodoV.anyadirHijo(nodoW);
        }
        
        /**
         * Divide el rbol que contiene el nodo recibido como parmetro en dos rboles
         * eliminando la arista saliente del nodo (todas las aristas estn dirigidas hacia
         * la raz) (se asume que el nodo no es la raz del rbol)
         * @param v un nodo
         */
        public void cut(int v) {
            Nodo nodoV = nodos.get(v);
            nodoV.getPadre().borrarHijo(nodoV);
        }
        
        /**
         * Divide el rbol que contiene dos nodos recibidos como parmetros en dos rboles
         * eliminando la arista incidente con los nodos en cuestin
         * @param v un nodo
         * @param w otro nodo
         */
        public void cut(int v, int w) {
            Nodo nodoV = nodos.get(v);
            Nodo nodoW = nodos.get(w);
            if (nodoV.getPadre() == nodoW)
                cut(v);
            else
                cut(w);
        }
        
        /**
         * Devuelve una copia de este bosque
         * @return una copia de este bosque
         */
        public Bosque copia() {
            ArrayList<Nodo> nodosCopia = new ArrayList<Nodo>();
            for (int i = 0; i < NUM_NODOS; i++) {
                Nodo nodo = nodos.get(i);
                if (nodo.getPadre() == null)
                    nodo.completarNodos(nodosCopia, null);
            }
            return new Bosque(nodosCopia);
        }
        
    }
    
    /**
     * Clase que representa un nodo del bosque generador que se mantendr durante la aplicacin
     * del algoritmo
     */
    class Nodo {
        
        private int id;
        private Nodo padre;
        private ArrayList<Nodo> hijos = new ArrayList<Nodo>();
        
        /**
         * Constructor
         * @param id identificador del nodo
         */
        public Nodo(int id) {
            this.id = id;
        }
        
        /**
         * Constructor
         * @param id identificador del nodo
         * @param padre padre del nodo (si null, se crea el nodo sin padre)
         */
        public Nodo(int id, Nodo padre) {
            this(id);
            this.padre = padre;
        }
        
        /**
         * Convierte este nodo en raz del rbol en el que se encuentra
         *
         */
        public void convertirEnRaiz() {
            if (padre != null) {
                Nodo nuevoHijo = padre;
                padre = null;
                hijos.add(nuevoHijo);
                nuevoHijo.convertirEnPadre(this);
            }
        }
        
        /**
         * Convierte el nodo recibido como parmetro en padre de este nodo (se asume
         * que el nodo recibido como parmetro es hijo de este nodo)
         * @param nodo un nodo
         */
        public void convertirEnPadre(Nodo nodo) {
            Nodo nuevoHijo = padre;
            int numHijos = hijos.size();
            for (int i = 0; i < numHijos; i++) {
                if (hijos.get(i) == nodo) {
                    hijos.remove(i);
                    break;
                }
            }
            padre = nodo;
            if (nuevoHijo != null) {
                hijos.add(nuevoHijo);
                nuevoHijo.convertirEnPadre(this);
            }
        }
        
        /**
         * Establece el padre de este nodo
         * @param padre el nuevo padre de este nodo
         */
        public void setPadre(Nodo padre) {
            this.padre = padre;
        }
        
        /**
         * Aade un nuevo hijo a la lista de hijos de este nodo
         * @param nodo el nuevo hijo
         */
        public void anyadirHijo(Nodo nodo) {
            hijos.add(nodo);
            nodo.setPadre(this);
        }
        
        /**
         * Devuelve el padre de este nodo
         * @return el padre de este nodo
         */
        public Nodo getPadre() {
            return padre;
        }
        
        /**
         * Si el nodo recibido como parmetro se encuentra en la lista de hijos de este nodo,
         * se borra de dicha lista
         * @param nodo un nodo
         */
        public void borrarHijo(Nodo nodo) {
            int numHijos = hijos.size();
            for (int i = 0; i < numHijos; i++) {
                if (hijos.get(i) == nodo) {
                    hijos.remove(i);
                    nodo.separarDePadre();
                    return;
                }
            }
        }
        
        /**
         * Separa este nodo de su padre
         *
         */
        public void separarDePadre() {
            padre = null;
        }
        
        /**
         * Devuelve la raz del rbol en el que se encuentra este nodo
         * @return la raz del rbol en el que se encuentra este nodo
         */
        public Nodo getRaiz() {
            if (padre == null)
                return this;
            return padre.getRaiz();
        }
        
        /**
         * Devuelve el identificador de este nodo
         * @return el identificador de este nodo
         */
        public int getId() {
            return id;
        }
        
        /**
         * Aade a la lista de nodos recibida como parmetro una copia de este nodo ms
         * una copia de cada uno de sus descendientes (directos e indirectos)
         * @param nodos lista de nodos
         * @param padreDeNodoCopia padre de la copia de este nodo (o null en caso de que el
         * nodo copia tenga que ser un nodo raz)
         * @return una copia de este nodo
         */
        public Nodo completarNodos(ArrayList<Nodo> nodos, Nodo padreDeNodoCopia) {
            Nodo nodoCopia = new Nodo(this.id, padreDeNodoCopia);
            insertar(nodos, nodoCopia);
            int numHijos = hijos.size();
            if (numHijos != 0) {
                ArrayList<Nodo> hijosCopia = new ArrayList<Nodo>();
                for (int i = 0; i < numHijos; i++)
                    hijosCopia.add(hijos.get(i).completarNodos(nodos, nodoCopia));
                nodoCopia.setHijos(hijosCopia);
            }
            return nodoCopia;
        }
        
        /**
         * Establece los hijos de este nodo
         * @param hijos los nuevos hijos de este nodo
         */
        public void setHijos(ArrayList<Nodo> hijos) {
            this.hijos = hijos;
        }
        
        /**
         * Inserta un nodo en una lista de nodos de forma que la lista se mantenga ordenada
         * de menor a mayor identificador de nodo
         * @param nodos lista de nodos
         * @param nodo nodo a insertar
         */
        private void insertar(ArrayList<Nodo> nodos, Nodo nodo) {
            int idNodo = nodo.getId();
            int numNodos = nodos.size();
            for (int i = 0; i < numNodos; i++) {
                if (idNodo < nodos.get(i).getId()) {
                    nodos.add(i, nodo);
                    return;
                }
            }
            nodos.add(nodo);
        }
        
    }
    
}
