package estructuras;

import java.awt.*;
import java.util.*;
import javax.swing.*;

import oyentes.*;

import interfaz.*;

/**
 * Clase de objetos capaces de aplicar el algoritmo de Kruskal para hallar un rbol
 * generador mximo
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class AplicadorKruskalMaximo extends AdaptadoraAplicadorAlgoritmo {

    private AreaDibujoGrafo areaDibujo;
    
    private PanelOpciones panelOpciones;
    private JPanel panelNombreAlgoritmo;
    private JButton volverAEdicionGrafo;
    
    private ArrayList<Nodo> nodos;
    private ArrayList<Arista> aristas;
    private int[] idsNodosOrigen;
    private int[] idsNodosDestino;
    private Vector<Integer> aristasNoEnArbol;
    private int numAristasEnArbol = 0;
    private int[] componentesConexas;
    private boolean[] enArbol; //indica, para cada arista del grafo, si la arista est en el rbol
    private final int NUM_NODOS;
    private final int NUM_ARISTAS;
    private static final int MIN_VALUE = 0;
    
    /**
     * Constructor
     * @param area rea de dibujo del grafo
     * @param panelOpciones panel de opciones disponibles durante la aplicacin de un algoritmo
     */
    public AplicadorKruskalMaximo(AreaDibujoGrafo area, PanelOpciones panelOpciones) {
        areaDibujo = area;
        this.panelOpciones = panelOpciones;
        nodos = area.getNodos();
        NUM_NODOS = nodos.size();
        aristas = area.getAristas();
        NUM_ARISTAS = aristas.size();
        idsNodosOrigen = area.getIdsNodosOrigen();
        idsNodosDestino = area.getIdsNodosDestino();
        inicializarAristasNoEnArbol();
        componentesConexas = new int[NUM_NODOS];
        for (int i = 0; i < NUM_NODOS; i++)
            componentesConexas[i] = i;
        enArbol = new boolean[NUM_ARISTAS];
        Arrays.fill(enArbol, false);
    }
    
    /**
     * Prepara el rea de dibujo teniendo en cuenta que se va a empezar
     * a aplicar el algoritmo para hallar un rbol generador.
     */
    public void prepararAreaDibujo() {
        if (NUM_ARISTAS == 0)
            areaDibujo.marcarNodo(0);
    }
    
    /**
     * Prepara el panel de opciones teniendo en cuenta que se va a empezar a aplicar el
     * algoritmo para hallar un rbol generador. Concretamente, establece el tamao del panel,
     * aade al panel los componentes (etiquetas, botones) que corresponda, registra en los
     * componentes los oyentes correspondientes y establece que el panel sea visible.
     * @param barra barra de mens (se necesitar para crear un oyente)
     */
    public void prepararPanelOpcionesPrincipio(BarraMenus barraMenus) {
        panelOpciones.setPreferredSize(new Dimension(260, 3000));
        
        panelNombreAlgoritmo = crearPanelNombreAlgoritmo(
                "<html><center>Maximum spanning tree:<br></center>" +
                "<center>Kruskal algorithm</center></html>");
        JButton construidoArbol = crearBoton(
                "<html><center>\"Maximum spanning tree<br></center>" +
                "<center>already found\"</center></html>", 2);
        construidoArbol.addActionListener(new OyenteTerminar(this));
        JButton solucionPasoProximo = crearBoton("Solution to next step");
        solucionPasoProximo.addActionListener(new OyenteSolucionPasoProximo(this));
        JButton solucionPasosRestantes = crearBoton("Solution to remaining steps");
        solucionPasosRestantes.addActionListener(new OyenteSolucionPasosRestantes(this));
        volverAEdicionGrafo = crearBoton("Back to graph edition");
        volverAEdicionGrafo.addActionListener(new OyenteVolverAEdicionGrafo(barraMenus,
                panelOpciones, areaDibujo));
        panelOpciones.add(panelNombreAlgoritmo);
        panelOpciones.add(construidoArbol);
        panelOpciones.add(solucionPasoProximo);
        panelOpciones.add(solucionPasosRestantes);
        panelOpciones.add(volverAEdicionGrafo);
        
        panelOpciones.setVisible(true);
    }
    
    /**
     * Prepara el panel de opciones teniendo en cuenta que ya se ha dado el
     * ltimo paso del algoritmo para hallar un rbol generador.
     */
    public void prepararPanelOpcionesFin() {
        panelOpciones.setVisible(false);
        panelOpciones.removeAll();
        panelOpciones.add(panelNombreAlgoritmo);
        panelOpciones.add(volverAEdicionGrafo);
        panelOpciones.setVisible(true);
    }
    
    /**
     * Crea un panel en el que se indicar qu algoritmo se aplica
     * @param texto texto a mostrar en el panel
     * @return el panel creado
     */
    private JPanel crearPanelNombreAlgoritmo(String texto) {
        JPanel panel = new JPanel();
        panel.setBackground(Color.BLACK);
        panel.setOpaque(true);
        panel.setPreferredSize(new Dimension(260, 70));
        panel.add(crearEtiqNombreAlgoritmo(texto));
        return panel;
    }
    
    /**
     * Crea una etiqueta en la que se indica qu algoritmo se aplica
     * @param texto el texto de la etiqueta
     * @return la etiqueta creada
     */
    private JLabel crearEtiqNombreAlgoritmo(String texto) {
        JLabel etiqueta = new JLabel(texto);
        etiqueta.setForeground(Color.WHITE);
        etiqueta.setHorizontalAlignment(SwingConstants.CENTER);
        etiqueta.setVerticalAlignment(SwingConstants.CENTER);
        
        //Se cambia el tamao de la fuente
        Font fuente = etiqueta.getFont();
        String nombreFuente = fuente.getName();
        int estiloFuente = fuente.getStyle();
        int tamaoFuente = fuente.getSize() + 2;
        etiqueta.setFont(new Font(nombreFuente, estiloFuente, tamaoFuente));
        
        etiqueta.setPreferredSize(new Dimension(250, 60));
        return etiqueta;
    }
    
    /**
     * Crea un botn con una lnea de texto para el panel de opciones
     * @param texto el texto del botn
     * @return el botn creada
     */
    private JButton crearBoton(String texto) {
        JButton boton = new JButton(texto);
        boton.setBorderPainted(true);
        boton.setPreferredSize(new Dimension(255, 25));
        return boton;
    }
    
    /**
     * Crea un botn para el panel de opciones
     * @param texto el texto del botn
     * @param numLineas el nmero de lneas del texto del botn
     * @return el botn creada
     */
    private JButton crearBoton(String texto, int numLineas) {
        JButton boton = new JButton(texto);
        boton.setBorderPainted(true);
        boton.setPreferredSize(new Dimension(255, 8 + (17 * numLineas)));
        return boton;
    }
    
    /**
     * Determina si el paso de elegir arista es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @param idArista la arista elegida
     * @return true si el paso es correcto; false en caso contrario
     */
    public boolean elegirArista(int idArista) {
        if (numAristasEnArbol == (NUM_NODOS - 1))
            return false;
        int pesoAristaSel = aristas.get(idArista).getPeso();
        if (enArbol[idArista])
            return false;
        ArrayList<Integer> aEliminar = new ArrayList<Integer>();
        boolean encontradaAnyadibleAlArbol = false;
        int pesoAnyadibleAlArbol = MIN_VALUE;
        int numVisitadas = 0;
        int idAristaActual;
        do {
            //En este punto no se ha visitado anteriormente la arista seleccionada.
            //S puede haberse visitado alguna arista no en rbol e incluso haberse
            //encontrado una arista no en rbol "aadible" al rbol
            int iAristaActual = aristasNoEnArbol.size() - 1 - numVisitadas;
            idAristaActual = aristasNoEnArbol.elementAt(iAristaActual).intValue();
            int pesoAristaActual = aristas.get(idAristaActual).getPeso();
            numVisitadas++;
            if (pesoAristaActual < pesoAristaSel)
                return false;
            if (encontradaAnyadibleAlArbol) {
                if (pesoAristaActual < pesoAnyadibleAlArbol)
                    return false;
                if (extremosEnMismaComponenteConexa(idAristaActual)) {
                    if (idAristaActual == idArista)
                        return false;
                    else
                        aEliminar.add(new Integer(iAristaActual));
                }
                else {
                    if (idAristaActual == idArista) {
                        anyadirAristaSelAArbol(idArista, aEliminar);
                        return true;
                    }
                }
            }
            else {  // (!encontradaAnyadibleAlArbol)
                if (extremosEnMismaComponenteConexa(idAristaActual)) {
                    if (idAristaActual == idArista)
                        return false;
                    else
                        aEliminar.add(new Integer(iAristaActual));
                }
                else {
                    if (idAristaActual == idArista) {
                        anyadirAristaSelAArbol(idArista, aEliminar);
                        return true;
                    }
                    else {
                        encontradaAnyadibleAlArbol = true;
                        pesoAnyadibleAlArbol = pesoAristaActual;
                    }
                }
            }
        } while (idAristaActual != idArista);
        return false;
    }
    
    /**
     * Determina si el paso de terminar es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @return true si el paso es correcto; false en caso contrario
     */
    public boolean terminar() {
        if (NUM_ARISTAS == 0) {
            prepararPanelOpcionesFin();
            return true;
        }
        if (numAristasEnArbol == (NUM_NODOS - 1)) {
            prepararPanelOpcionesFin();
            return true;
        }
        return false;
    }
    
    /**
     * Introduce las modificaciones oportunas en el area de dibujo teniendo en cuenta
     * que en dicho area se desea que quede reflejado el resultado de aplicar un paso
     * correcto del algoritmo.
     * Adems, actualiza el panel de opciones.
     */
    public void solucionPasoProximo() {
        if (NUM_ARISTAS == 0) {
            prepararPanelOpcionesFin();
            return;
        }
        if (numAristasEnArbol == (NUM_NODOS - 1)) {
            prepararPanelOpcionesFin();
            return;
        }
        int idArista;
        do {
            idArista = aristasNoEnArbol.remove(aristasNoEnArbol.size() - 1).intValue();
        } while (extremosEnMismaComponenteConexa(idArista));
        numAristasEnArbol++;
        int idOrigen = idsNodosOrigen[idArista];
        int idDestino = idsNodosDestino[idArista];
        actualizarComponentesConexas(idOrigen, idDestino);
        enArbol[idArista] = true;
        areaDibujo.marcarNodo(idOrigen);
        areaDibujo.marcarNodo(idDestino);
        areaDibujo.marcarArista(idArista);
    }
    
    /**
     * Introduce las modificaciones oportunas en el area de dibujo teniendo en cuenta
     * que en dicho area se desea que quede reflejado el resultado de aplicar pasos
     * correctos del algoritmo hasta construir el rbol generador completo.
     * Adems, actualiza el panel de opciones.
     */
    public void solucionPasosRestantes() {
        if (NUM_ARISTAS == 0) {
            prepararPanelOpcionesFin();
            return;
        }
        while (numAristasEnArbol < (NUM_NODOS - 1)) {
            int idArista = aristasNoEnArbol.remove(aristasNoEnArbol.size() - 1).intValue();
            if (extremosEnMismaComponenteConexa(idArista))
                continue;
            numAristasEnArbol++;
            int idOrigen = idsNodosOrigen[idArista];
            int idDestino = idsNodosDestino[idArista];
            actualizarComponentesConexas(idOrigen, idDestino);
            enArbol[idArista] = true;
            areaDibujo.marcarNodo(idOrigen);
            areaDibujo.marcarNodo(idDestino);
            areaDibujo.marcarArista(idArista);
        }
        prepararPanelOpcionesFin();
    }
    
    /**
     * Indica cul es la salida del algoritmo.
     * @return una cadena que indica cul es la salida del algoritmo.
     */
    public String salida() {
        return "maximum spanning tree";
    }
    
    /**
     * Inicializa la variable aristasNoEnArbol (lista de identificadores de aristas no
     * pertenecientes al rbol generador)
     */
    private void inicializarAristasNoEnArbol() {
        aristasNoEnArbol = new Vector<Integer>();
        int numAristasNoEnArbol = 0;
        for (int idAristaAInsertar = 0; idAristaAInsertar < NUM_ARISTAS; idAristaAInsertar++) {
            boolean insertado = false;
            int pesoAristaAInsertar = aristas.get(idAristaAInsertar).getPeso();
            for (int i = 0; i < numAristasNoEnArbol; i++) {
                if (pesoAristaAInsertar <
                        aristas.get(aristasNoEnArbol.elementAt(i).intValue()).getPeso()) {
                    aristasNoEnArbol.insertElementAt(new Integer(idAristaAInsertar), i);
                    insertado = true;
                    break;
                }
            }
            if (!insertado)
                aristasNoEnArbol.add(new Integer(idAristaAInsertar));
            numAristasNoEnArbol++;
        }
    }
    
    /**
     * Actualiza el array que indica a qu componente conexa pertenece cada nodo del
     * grafo teniendo en cuenta que dos nodos pasan a pertenecer a la misma componente
     * conexa
     * @param idNodo1 identificador de uno de los dos nodos en cuestin
     * @param idNodo2 identificador del otro nodo
     */
    private void actualizarComponentesConexas(int idNodo1, int idNodo2) {
        int compSustituta = componentesConexas[idNodo1];
        int compASustituir = componentesConexas[idNodo2];
        for (int i = 0; i < componentesConexas.length; i++) {
            if (componentesConexas[i] == compASustituir)
                componentesConexas[i] = compSustituta;
        }
    }
    
    /**
     * Determina si los nodos extremos de una arista pertenecen a una misma
     * componente conexa
     * @param idArista identificador de la arista
     * @return true en caso positivo; false en caso contrario
     */
    private boolean extremosEnMismaComponenteConexa(int idArista) {
        return componentesConexas[idsNodosOrigen[idArista]] == 
            componentesConexas[idsNodosDestino[idArista]];
    }
    
    /**
     * Aade al rbol una arista seleccionada por el usuario y realiza todas las acciones
     * que tiene que realizar el mtodo anyadirArista de esta clase antes de devolver true
     * @param idAristaSel la arista seleccionada
     * @param aEliminar indices en la lista de aristas no en rbol de las aristas a
     * eliminar de dicha lista
     */
    private void anyadirAristaSelAArbol(int idAristaSel, ArrayList<Integer> aEliminar) {
        numAristasEnArbol++;
        int idOrigen = idsNodosOrigen[idAristaSel];
        int idDestino = idsNodosDestino[idAristaSel];
        actualizarComponentesConexas(idOrigen, idDestino);
        enArbol[idAristaSel] = true;
        for (int i = 0; i < aEliminar.size(); i++)
            aristasNoEnArbol.remove(aEliminar.get(i).intValue());
        areaDibujo.marcarNodo(idOrigen);
        areaDibujo.marcarNodo(idDestino);
        areaDibujo.marcarArista(idAristaSel);
    }
    
}
