package estructuras;

import interfaz.*;

import java.awt.*;
import java.util.*;
import javax.swing.*;

import oyentes.*;

/**
 * Clase de objetos capaces de aplicar el algoritmo de Prim para hallar un rbol
 * generador mnimo
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class AplicadorPrimMinimo extends AdaptadoraAplicadorAlgoritmo {

    private AreaDibujoGrafo areaDibujo;
    
    private PanelOpciones panelOpciones;
    private JPanel panelNombreAlgoritmo;
    private JButton volverAEdicionGrafo;
    
    private ArrayList<Nodo> nodos;
    private ArrayList<Arista> aristas;
    private int[][] adyacencias;
    private int[] idsNodosOrigen;
    private int[] idsNodosDestino;
    private ArrayList<Integer> nodosEnArbol = new ArrayList<Integer>();
    private ArrayList<Integer> nodosNoEnArbol = new ArrayList<Integer>();
    private ArrayList<Integer> etiquetasNodosNoEnArbol = new ArrayList<Integer>();
    private int etiquetaMinima;
    private boolean[] enArbol;  //indica, para cada nodo del grafo, si el nodo est en el rbol
    private final int NUM_NODOS;
    
    /**
     * Constructor
     * @param area rea de dibujo del grafo
     * @param panelOpciones panel de opciones disponibles durante la aplicacin de un algoritmo
     */
    public AplicadorPrimMinimo(AreaDibujoGrafo area, PanelOpciones panelOpciones) {
        areaDibujo = area;
        this.panelOpciones = panelOpciones;
        nodos = area.getNodos();
        aristas = area.getAristas();
        adyacencias = area.getAdyacencias();
        idsNodosOrigen = area.getIdsNodosOrigen();
        idsNodosDestino = area.getIdsNodosDestino();
        NUM_NODOS = nodos.size();
        for (int i = 0; i < NUM_NODOS; i++) {
            nodosNoEnArbol.add(new Integer(i));
            etiquetasNodosNoEnArbol.add(new Integer(Integer.MAX_VALUE));
        }
        etiquetaMinima = Integer.MAX_VALUE;
        enArbol = new boolean[NUM_NODOS];
        Arrays.fill(enArbol, false);
    }
    
    /**
     * Prepara el panel de opciones teniendo en cuenta que se va a empezar a aplicar el
     * algoritmo para hallar un rbol generador. Concretamente, establece el tamao del panel,
     * aade al panel los componentes (etiquetas, botones) que corresponda, registra en los
     * componentes los oyentes correspondientes y establece que el panel sea visible.
     * @param barra barra de mens (se necesitar para crear un oyente)
     */
    public void prepararPanelOpcionesPrincipio(BarraMenus barraMenus) {
        panelOpciones.setPreferredSize(new Dimension(260, 3000));
        
        panelNombreAlgoritmo = crearPanelNombreAlgoritmo(
                "<html><center>Minimum spanning tree:<br></center>" +
                "<center>Prim algorithm</center></html>");
        JButton construidoArbol = crearBoton(
                "<html><center>\"Minimum spanning tree<br></center>" +
                "<center>already found\"</center></html>", 2);
        construidoArbol.addActionListener(new OyenteTerminar(this));
        JButton solucionPasoProximo = crearBoton("Solution to next step");
        solucionPasoProximo.addActionListener(new OyenteSolucionPasoProximo(this));
        JButton solucionPasosRestantes = crearBoton("Solution to remaining steps");
        solucionPasosRestantes.addActionListener(new OyenteSolucionPasosRestantes(this));
        volverAEdicionGrafo = crearBoton("Back to graph edition");
        volverAEdicionGrafo.addActionListener(new OyenteVolverAEdicionGrafo(barraMenus,
                panelOpciones, areaDibujo));
        panelOpciones.add(panelNombreAlgoritmo);
        panelOpciones.add(construidoArbol);
        panelOpciones.add(solucionPasoProximo);
        panelOpciones.add(solucionPasosRestantes);
        panelOpciones.add(volverAEdicionGrafo);
        
        panelOpciones.setVisible(true);
    }
    
    /**
     * Prepara el panel de opciones teniendo en cuenta que ya se ha dado el
     * ltimo paso del algoritmo para hallar un rbol generador.
     */
    public void prepararPanelOpcionesFin() {
        panelOpciones.setVisible(false);
        panelOpciones.removeAll();
        panelOpciones.add(panelNombreAlgoritmo);
        panelOpciones.add(volverAEdicionGrafo);
        panelOpciones.setVisible(true);
    }
    
    /**
     * Crea un panel en el que se indicar qu algoritmo se aplica
     * @param texto texto a mostrar en el panel
     * @return el panel creado
     */
    private JPanel crearPanelNombreAlgoritmo(String texto) {
        JPanel panel = new JPanel();
        panel.setBackground(Color.BLACK);
        panel.setOpaque(true);
        panel.setPreferredSize(new Dimension(260, 70));
        panel.add(crearEtiqNombreAlgoritmo(texto));
        return panel;
    }
    
    /**
     * Crea una etiqueta en la que se indica qu algoritmo se aplica
     * @param texto el texto de la etiqueta
     * @return la etiqueta creada
     */
    private JLabel crearEtiqNombreAlgoritmo(String texto) {
        JLabel etiqueta = new JLabel(texto);
        etiqueta.setForeground(Color.WHITE);
        etiqueta.setHorizontalAlignment(SwingConstants.CENTER);
        etiqueta.setVerticalAlignment(SwingConstants.CENTER);
        
        //Se cambia el tamao de la fuente
        Font fuente = etiqueta.getFont();
        String nombreFuente = fuente.getName();
        int estiloFuente = fuente.getStyle();
        int tamaoFuente = fuente.getSize() + 2;
        etiqueta.setFont(new Font(nombreFuente, estiloFuente, tamaoFuente));
        
        etiqueta.setPreferredSize(new Dimension(250, 60));
        return etiqueta;
    }
    
    /**
     * Crea un botn con una lnea de texto para el panel de opciones
     * @param texto el texto del botn
     * @return el botn creada
     */
    private JButton crearBoton(String texto) {
        JButton boton = new JButton(texto);
        boton.setBorderPainted(true);
        boton.setPreferredSize(new Dimension(255, 25));
        return boton;
    }
    
    /**
     * Crea un botn para el panel de opciones
     * @param texto el texto del botn
     * @param numLineas el nmero de lneas del texto del botn
     * @return el botn creada
     */
    private JButton crearBoton(String texto, int numLineas) {
        JButton boton = new JButton(texto);
        boton.setBorderPainted(true);
        boton.setPreferredSize(new Dimension(255, 8 + (17 * numLineas)));
        return boton;
    }
    
    /**
     * Determina si el paso de elegir vrtice es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @param idVertice el vrtice elegido
     * @return true si el paso es correcto; false en caso contrario
     */
    public boolean elegirVertice(int idVertice) {
        if (nodosEnArbol.size() > 0)
            return false;
        anyadirNodoAArbol(idVertice);
        areaDibujo.marcarNodo(idVertice);
        return true;
    }
    
    /**
     * Determina si el paso de elegir arista es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @param idArista la arista elegida
     * @return true si el paso es correcto; false en caso contrario
     */
    public boolean elegirArista(int idArista) {
        int numNodosEnArbol = nodosEnArbol.size();
        if ((numNodosEnArbol == 0) || (numNodosEnArbol == NUM_NODOS))
            return false;
        
        //Se comprueba si el peso de la arista coincide con la etiqueta mnima
        int pesoArista = aristas.get(idArista).getPeso();
        if (pesoArista != etiquetaMinima)
            return false;
        
        //Se comprueba si uno de los nodos incidentes con la arista pertenece al rbol
        //y el otro no pertenece al rbol
        int idOrigen = idsNodosOrigen[idArista];
        int idDestino = idsNodosDestino[idArista];
        if (!((enArbol[idOrigen] && !enArbol[idDestino]) ||
                (!enArbol[idOrigen] && enArbol[idDestino])))
            return false;
        
        int idNodoAAnyadir;
        if (!enArbol[idOrigen])
            idNodoAAnyadir = idOrigen;
        else
            idNodoAAnyadir = idDestino;
        int numNodosNoEnArbol = nodosNoEnArbol.size();
        int iNodoAAnyadir = -1;
        for (int i = 0; i < numNodosNoEnArbol; i++) {
            if (nodosNoEnArbol.get(i).intValue() == idNodoAAnyadir) {
                iNodoAAnyadir = i;
                break;
            }
        }
        anyadirNodoAArbol(iNodoAAnyadir);
        areaDibujo.marcarNodo(idNodoAAnyadir);
        areaDibujo.marcarArista(idArista);
        return true;
    }
    
    /**
     * Determina si el paso de terminar es correcto y, en caso positivo, introduce
     * las modificaciones oportunas en el area de dibujo y actualiza el panel de opciones
     * @return true si el paso es correcto; false en caso contrario
     */
    public boolean terminar() {
        if (nodosEnArbol.size() != NUM_NODOS)
            return false;
        prepararPanelOpcionesFin();
        return true;
    }
    
    /**
     * Introduce las modificaciones oportunas en el area de dibujo teniendo en cuenta
     * que en dicho area se desea que quede reflejado el resultado de aplicar un paso
     * correcto del algoritmo.
     * Adems, actualiza el panel de opciones.
     */
    public void solucionPasoProximo() {
        if (nodosEnArbol.size() == NUM_NODOS) {
            prepararPanelOpcionesFin();
            return;
        }
        if (nodosEnArbol.size() == 0) {
            int i = 0;
            int idNodoAAnyadir = nodosNoEnArbol.get(i).intValue();
            anyadirNodoAArbol(i);
            areaDibujo.marcarNodo(idNodoAAnyadir);
        }
        else {
            
            //Se busca un nodo no en rbol con etiqueta mnima
            int numNodosNoEnArbol = nodosNoEnArbol.size();
            int iNodoAAnyadir = -1;
            for (int i = 0; i < numNodosNoEnArbol; i++) {
                int etiquetaActual = etiquetasNodosNoEnArbol.get(i).intValue();
                if (etiquetaActual == etiquetaMinima) {
                    iNodoAAnyadir = i;
                    break;
                }
            }
            int idNodoAAnyadir = nodosNoEnArbol.get(iNodoAAnyadir).intValue();
            
            //Se busca una arista cuyo peso coincida con la etiqueta mnima que una el
            //nodo encontrado con un nodo del rbol
            int numNodosEnArbol = nodosEnArbol.size();
            int idAristaAAnyadir = -1;
            for (int i = 0; i < numNodosEnArbol; i++) {
                int idNodoEnArbol = nodosEnArbol.get(i).intValue();
                int idArista = adyacencias[idNodoAAnyadir][idNodoEnArbol];
                if (idArista >= 0) {
                    if (aristas.get(idArista).getPeso() == etiquetaMinima) {
                        idAristaAAnyadir = idArista;
                        break;
                    }
                }
            }
            
            anyadirNodoAArbol(iNodoAAnyadir);
            areaDibujo.marcarNodo(idNodoAAnyadir);
            areaDibujo.marcarArista(idAristaAAnyadir);
        }
    }
    
    
    /**
     * Introduce las modificaciones oportunas en el area de dibujo teniendo en cuenta
     * que en dicho area se desea que quede reflejado el resultado de aplicar pasos
     * correctos del algoritmo hasta construir el rbol generador completo.
     * Adems, actualiza el panel de opciones.
     */
    public void solucionPasosRestantes() {
        if (nodosEnArbol.size() == 0) {
            int i = 0;
            int idNodoAAnyadir = nodosNoEnArbol.get(i).intValue();
            anyadirNodoAArbol(i);
            areaDibujo.marcarNodo(idNodoAAnyadir);
        }
        while (nodosEnArbol.size() < NUM_NODOS) {
            
            //Se busca un nodo no en rbol con etiqueta mnima
            int numNodosNoEnArbol = nodosNoEnArbol.size();
            int iNodoAAnyadir = -1;
            for (int i = 0; i < numNodosNoEnArbol; i++) {
                int etiquetaActual = etiquetasNodosNoEnArbol.get(i).intValue();
                if (etiquetaActual == etiquetaMinima) {
                    iNodoAAnyadir = i;
                    break;
                }
            }
            int idNodoAAnyadir = nodosNoEnArbol.get(iNodoAAnyadir).intValue();
            
            //Se busca una arista cuyo peso coincida con la etiqueta mnima que una el
            //nodo encontrado con un nodo del rbol
            int numNodosEnArbol = nodosEnArbol.size();
            int idAristaAAnyadir = -1;
            for (int i = 0; i < numNodosEnArbol; i++) {
                int idNodoEnArbol = nodosEnArbol.get(i).intValue();
                int idArista = adyacencias[idNodoAAnyadir][idNodoEnArbol];
                if (idArista >= 0) {
                    if (aristas.get(idArista).getPeso() == etiquetaMinima) {
                        idAristaAAnyadir = idArista;
                        break;
                    }
                }
            }
            
            anyadirNodoAArbol(iNodoAAnyadir);
            areaDibujo.marcarNodo(idNodoAAnyadir);
            areaDibujo.marcarArista(idAristaAAnyadir);
        }
        prepararPanelOpcionesFin();
    }
    
    /**
     * Indica cul es la salida del algoritmo.
     * @return una cadena que indica cul es la salida del algoritmo.
     */
    public String salida() {
        return "minimum spanning tree";
    }
    
    /**
     * Pasa un nodo de la lista de nodos no en rbol a la lista de nodos en rbol y
     * actualiza el array que indica qu nodos estn en el rbol, la lista de etiquetas
     * de los nodos no en rbol y la variable que contiene el valor de la etiqueta mnima
     * @param i indice para acceder al nodo dentro de la lista de nodos no en rbol
     */
    private void anyadirNodoAArbol(int i) {
        int idNodo = nodosNoEnArbol.get(i).intValue();
        nodosEnArbol.add(nodosNoEnArbol.remove(i));
        etiquetasNodosNoEnArbol.remove(i);
        int numNodosNoEnArbol = nodosNoEnArbol.size();
        etiquetaMinima = Integer.MAX_VALUE;
        for (int iNodoNoEnArbol = 0; iNodoNoEnArbol < numNodosNoEnArbol; iNodoNoEnArbol++) {
            int idArista = adyacencias[idNodo][nodosNoEnArbol.get(iNodoNoEnArbol).intValue()];
            if (idArista >= 0) {
                int nuevaEtiqueta = aristas.get(idArista).getPeso();
                if (nuevaEtiqueta < etiquetasNodosNoEnArbol.get(iNodoNoEnArbol).intValue())
                    etiquetasNodosNoEnArbol.set(iNodoNoEnArbol, new Integer(nuevaEtiqueta));
            }
            int etiquetaActual = etiquetasNodosNoEnArbol.get(iNodoNoEnArbol).intValue();
            if (etiquetaActual < etiquetaMinima)
                etiquetaMinima = etiquetaActual;
        }
        enArbol[idNodo] = true;
    }
    
}
