package estructuras;

import java.awt.Color;
import java.awt.Graphics;

/**
 * Clase de objetos que representan aristas
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class Arista {

    private static final Color COLOR_POR_DEFECTO = Color.blue;
    private static final Color COLOR_SELECCIONADO = new Color(191, 191, 0);
    private static final Color COLOR_ARBOL = Color.red;
    private static final Color COLOR_ARBOL_AUX = Color.green;
    private static final Color COLOR_POR_DEFECTO_OSCURO = new Color(0, 0, 0);
    private static final Color COLOR_SELECCIONADO_OSCURO = new Color(127, 127, 0);
    private static final Color COLOR_ARBOL_OSCURO = new Color(127, 0, 0);
    private static final Color COLOR_ARBOL_AUX_OSCURO = new Color(0, 127, 0);
    
    private Nodo nodoOrigen;
    private Nodo nodoDestino;
    private int peso;
    private String etiqueta;
    boolean seleccionada = false;
    boolean perteneceAArbol = false;
    boolean perteneceAArbolAux = false;
    boolean etiquetada = false;
    
    /**
     * Constructor
     * @param origen nodo origen de la arista
     * @param destino nodo destino de la arista
     */
    public Arista(Nodo origen, Nodo destino) {
        nodoOrigen = origen;
        nodoDestino = destino;
        peso = 1;
    }
    
    /**
     * Constructor
     * @param origen nodo origen de la arista
     * @param destino nodo destino de la arista
     * @param peso peso de la arista
     */
    public Arista(Nodo origen, Nodo destino, int peso) {
        nodoOrigen = origen;
        nodoDestino = destino;
        this.peso = peso;
    }
    
    /**
     * Devuelve el nodo origen de esta arista
     * @return el nodo origen de esta arista
     */
    public Nodo getNodoOrigen() {
        return nodoOrigen;
    }

    /**
     * Devuelve el nodo destino de esta arista
     * @return el nodo destino de esta arista
     */
    public Nodo getNodoDestino() {
        return nodoDestino;
    }

    /**
     * Devuelve el peso de esta arista
     * @return el peso de esta arista
     */
    public int getPeso() {
        return peso;
    }

    /**
     * Determina si esta arista est seleccionada
     * @return true si esta arista est seleccionada; false en caso contrario
     */
    public boolean estaSeleccionada() {
        return seleccionada;
    }
    
    /**
     * Selecciona o deselecciona esta arista
     * @param seleccionada indica si hay que seleccionar la arista (true) o deseleccionarla
     * (false)
     */
    public void setSeleccionada(boolean seleccionada) {
        this.seleccionada = seleccionada;
    }
    
    /**
     * Determina si un punto es de la arista
     * @param xPunto coordenada X del punto
     * @param yPunto coordenada Y del punto
     * @return true si el punto es de la arista; false en caso contrario
     */
    public boolean esAristaBuscada(int xPunto, int yPunto) {
        final int DISTANCIA_MAX = 5;
        int xOrigen = nodoOrigen.getX();
        int yOrigen = nodoOrigen.getY();
        int xDestino = nodoDestino.getX();
        int yDestino = nodoDestino.getY();
        
        // Si la arista es "vertical"
        if (xOrigen == xDestino)
            return (xPunto >= xOrigen - DISTANCIA_MAX) &&
                    (xPunto <= xOrigen + DISTANCIA_MAX) &&
                    !((yPunto > yOrigen) && (yPunto > yDestino)) &&
                    !((yPunto < yOrigen) && (yPunto < yDestino));
        
        // Si la arista es "horizontal"
        if (yOrigen == yDestino)
            return (yPunto >= yOrigen - DISTANCIA_MAX) &&
                    (yPunto <= yOrigen + DISTANCIA_MAX) &&
                    !((xPunto > xOrigen) && (xPunto > xDestino)) &&
                    !((xPunto < xOrigen) && (xPunto < xDestino));
        
        // Caso general (la arista no es "vertical" ni "horizontal")
        
        /*
         * Obtener ecuacin de la recta Recta que contiene la arista, de la forma
         * Y = M*X + B  (obtener concretamente M y B)
         */
        float mRecta = (yDestino - yOrigen) / ((float)(xDestino - xOrigen));
        float bRecta = yOrigen - (xOrigen * mRecta);
        
        /*
         * Obtener ecuacin de la recta Recta2 perpendicular a Recta que pasa por Punto,
         * de la forma  Y = M*X + B  (obtener concretamente M y B)
         */
        float mRecta2 = -(1 / mRecta);
        float bRecta2 = yPunto - (mRecta2 * xPunto);
        
        // Obtener punto Punto2 de interseccin de las dos rectas
        int xPunto2 = Math.round((bRecta2 - bRecta) / (mRecta - mRecta2));
        int yPunto2 = Math.round((mRecta * ((bRecta2 - bRecta) / (mRecta - mRecta2))) +
                bRecta);
        
        // Obtener la distancia al cuadrado de Punto a Punto2
        int cuadradoDeDistancia = ((xPunto - xPunto2) * (xPunto - xPunto2)) +
        ((yPunto - yPunto2) * (yPunto - yPunto2));
        
        /*
         * Devolver false si la distancia de Punto a Punto2 es mayor que la
         * distancia DISTANCIA_MAX
         */
        int cuadradoDeDistanciaMax = DISTANCIA_MAX * DISTANCIA_MAX;
        if (cuadradoDeDistancia > cuadradoDeDistanciaMax)
            return false;
        
        // Devolver false, si Punto2 no pertenece a la arista, o true, en caso contrario
        if (mRecta <= 1)
            return !(((xPunto2 > xOrigen) && (xPunto2 > xDestino)) ||
                    ((xPunto2 < xOrigen) && (xPunto2 < xDestino)));
        else
            return !(((yPunto2 > yOrigen) && (yPunto2 > yDestino)) ||
                    ((yPunto2 < yOrigen) && (yPunto2 < yDestino)));
    }
    
    /**
     * Determina si esta arista es incidente con los dos nodos recibidos como parmetros
     * @param nodo1 uno de los dos nodos
     * @param nodo2 el otro nodo
     * @return true en caso positivo; false en caso contrario
     */
    public boolean incidenteCon(Nodo nodo1, Nodo nodo2) {
        return ((nodo1 == nodoOrigen) && (nodo2 == nodoDestino)) ||
        ((nodo2 == nodoOrigen) && (nodo1 == nodoDestino));
    }
    
    /**
     * Determina si esta arista es incidente con el nodo recibido como parmetro
     * @param nodo un nodo
     * @return true en caso positivo; false en caso contrario
     */
    public boolean incidenteCon(Nodo nodo) {
        return (nodo == nodoOrigen) || (nodo == nodoDestino);
    }
    
    /**
     * Pinta esta arista
     * @param g contexto grfico para pintar la arista
     */
    public void pintar(Graphics g) {
        if (seleccionada)
            g.setColor(COLOR_SELECCIONADO);
        else if (perteneceAArbol)
            g.setColor(COLOR_ARBOL);
        else if (perteneceAArbolAux)
            g.setColor(COLOR_ARBOL_AUX);
        else
            g.setColor(COLOR_POR_DEFECTO);
        int xOrigen = nodoOrigen.getX();
        int yOrigen = nodoOrigen.getY();
        int xDestino = nodoDestino.getX();
        int yDestino = nodoDestino.getY();
        g.drawLine(xOrigen, yOrigen, xDestino, yDestino);
        
        nodoOrigen.pintar(g);
        nodoDestino.pintar(g);
    }
    
    /**
     * Pinta el texto asociado a esta arista (la etiqueta y/o el peso de la arista)
     * @param g contexto grfico para pintar el texto
     */
    public void pintarTexto(Graphics g) {
        int xOrigen = nodoOrigen.getX();
        int yOrigen = nodoOrigen.getY();
        int xDestino = nodoDestino.getX();
        int yDestino = nodoDestino.getY();
        if (seleccionada)
            g.setColor(COLOR_SELECCIONADO_OSCURO);
        else if (perteneceAArbol)
            g.setColor(COLOR_ARBOL_OSCURO);
        else if (perteneceAArbolAux)
            g.setColor(COLOR_ARBOL_AUX_OSCURO);
        else
            g.setColor(COLOR_POR_DEFECTO_OSCURO);
        int x = xDePeso(xOrigen, xDestino);
        int y = yDePeso(yOrigen, yDestino);
        if (etiquetada)
            g.drawString(etiqueta + " (" + peso + ")", x, y);
        else
            g.drawString(String.valueOf(peso), x, y);
    }
    
    /**
     * Devuelve la coordenada X que se utilizar para pintar el peso de una arista
     * dadas las coordenadas X de los nodos origen y destino de la arista en cuestin
     * @param xOrigen coordenada X del nodo origen
     * @param xDestino coordenada X del nodo destino
     * @return la coordenada X que se utilizar para pintar el peso de la arista
     */
    private int xDePeso(int xOrigen, int xDestino) {
        if (xOrigen < xDestino)
            return (xOrigen + ((xDestino - xOrigen)/2));
        else
            return (xDestino + ((xOrigen - xDestino)/2));
    }
    
    /**
     * Devuelve la coordenada Y que se utilizar para pintar el peso de una arista
     * dadas las coordenadas Y de los nodos origen y destino de la arista en cuestin
     * @param xOrigen coordenada Y del nodo origen
     * @param xDestino coordenada Y del nodo destino
     * @return la coordenada Y que se utilizar para pintar el peso de la arista
     */
    private int yDePeso(int yOrigen, int yDestino) {
        if (yOrigen < yDestino)
            return (yOrigen + ((yDestino - yOrigen) / 2));
        else
            return (yDestino + ((yOrigen - yDestino) / 2));
    }
    
    /**
     * Cambia el peso de esta arista
     * @param nuevoPeso el nuevo peso de esta arista
     */
    public void cambiarPeso(int nuevoPeso) {
        peso = nuevoPeso;
    }
    
    /**
     * Determina si esta arista pertenece a un rbol generador construido o en construccin
     * @return true en caso positivo; false en caso contrario
     */
    public boolean perteneceAArbol() {
        return perteneceAArbol;
    }
    
    /**
     * Marca o desmarca esta arista como perteneciente a un rbol generador construido o
     * en construccin
     * @param pertenece indica si hay que marcar la arista (true) o desmarcarla (false)
     */
    public void setPerteneceAArbol(boolean pertenece) {
        perteneceAArbol = pertenece;
    }
    
    /**
     * Determina si esta arista pertenece a un rbol auxiliar
     * @return true en caso positivo; false en caso contrario
     */
    public boolean perteneceAArbolAux() {
        return perteneceAArbolAux;
    }
    
    /**
     * Marca o desmarca esta arista como perteneciente a un rbol auxiliar
     * @param pertenece indica si hay que marcar la arista (true) o desmarcarla (false)
     */
    public void setPerteneceAArbolAux(boolean pertenece) {
        perteneceAArbolAux = pertenece;
    }
    
    /**
     * Etiqueta esta arista
     * @param etiqueta nueva etiqueta para esta arista
     */
    public void etiquetar(String etiqueta) {
        this.etiqueta = etiqueta;
        etiquetada = true;
    }
    
    /**
     * Desetiqueta esta arista
     */
    public void desetiquetar() {
        etiquetada = false;
    }
    
}
