package estructuras;

import java.util.*;
import java.awt.*;
import java.io.*;

/**
 * Clase de objetos que representan grafos
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class Grafo {

    private ArrayList<Nodo> nodos = new ArrayList<Nodo>();
    private ArrayList<Arista> aristas = new ArrayList<Arista>();
    private ArrayList<Nodo> nodosSeleccionados = new ArrayList<Nodo>();
    private ArrayList<Arista> aristasSeleccionadas = new ArrayList<Arista>();
    private ArrayList<Nodo> nodosDeArbol = new ArrayList<Nodo>();
    private ArrayList<Arista> aristasDeArbol = new ArrayList<Arista>();
    private ArrayList<Nodo> nodosDeArbolAux = new ArrayList<Nodo>();
    private ArrayList<Arista> aristasDeArbolAux = new ArrayList<Arista>();
    private int[][] adyacencias;
    private int[] idsNodosOrigen;
    private int[] idsNodosDestino;
    private Preferencias preferencias;
    
    /**
     * Constructor
     * @param preferencias preferencias del usuario
     */
    public Grafo(Preferencias preferencias) {
        super();
        this.preferencias = preferencias;
    }
    
    /**
     * Constructor
     * @param cadena cadena de caracteres que representa un grafo
     */
    public Grafo(String cadena, Preferencias preferencias) {
        this.preferencias = preferencias;
        BufferedReader flujo = new BufferedReader(new StringReader(cadena));
        try {
            flujo.readLine();
            String linea = flujo.readLine();
            if (linea.compareTo("") == 0) {
                flujo.close();
                return;
            }
            String[] datosNodos = aArray(linea);
            for (int i = 1; i < datosNodos.length; i = i + 2)
                crearNodo(Integer.parseInt(datosNodos[i - 1]), Integer.parseInt(datosNodos[i]));
            linea = flujo.readLine();
            if (linea.compareTo("") == 0) {
                flujo.close();
                return;
            }
            String[] datosAristas = aArray(linea);
            for (int i = 2; i < datosAristas.length; i = i + 3) {
                Nodo nodoOrigen = nodos.get(Integer.parseInt(datosAristas[i - 2]));
                Nodo nodoDestino = nodos.get(Integer.parseInt(datosAristas[i - 1]));
                int peso = Integer.parseInt(datosAristas[i]);
                aristas.add(new Arista(nodoOrigen, nodoDestino, peso));
            }
            flujo.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Devuelve las aristas de este grafo
     * @return las aristas de este grafo
     */
    public ArrayList<Arista> getAristas() {
        return aristas;
    }

    /**
     * Actualiza la lista de aristas de este grafo
     * @param aristas la nueva lista de aristas
     */
    public void setAristas(ArrayList<Arista> aristas) {
        this.aristas = aristas;
    }

    /**
     * Devuelve los nodos de este grafo
     * @return los nodos de este grafo
     */
    public ArrayList<Nodo> getNodos() {
        return nodos;
    }

    /**
     * Actualiza la lista de nodos de este grafo
     * @param nodos la nueva lista de nodos
     */
    public void setNodos(ArrayList<Nodo> nodos) {
        this.nodos = nodos;
    }
    
    /**
     * Busca un nodo en la lista de nodos de este grafo
     * @param nodoBuscado el nodo buscado
     * @return indice del nodo en la lista de nodos; devuelve -1 si no se encuentra el nodo
     */
    private int indiceEnNodosDe(Nodo nodoBuscado) {
        Iterator<Nodo> itNodos = nodos.iterator();
        int i = 0;
        while (itNodos.hasNext()) {
            Nodo n = itNodos.next();
            if (n == nodoBuscado)
                return i;
            i++;
        }
        return -1;
    }
    
    /**
     * Busca una arista en la lista de aristas de este grafo
     * @param aristaBuscada la arista buscada
     * @return indice de la arista en la lista de aristas; devuelve -1 si no se encuentra
     * la arista
     */
    private int indiceEnAristasDe(Arista aristaBuscada) {
        Iterator<Arista> itAristas = aristas.iterator();
        int i = 0;
        while (itAristas.hasNext()) {
            Arista a = itAristas.next();
            if (a == aristaBuscada)
                return i;
            i++;
        }
        return -1;
    }
    
    /**
     * Convierte una cadena de datos para reconstruir un grafo a un array de subcadenas
     * de la cadena en cuestin, donde cada subcadena representa un dato.
     * @param cadenaDatos la cadena a convertir
     * @return el array resultante
     */
    private String[] aArray(String cadenaDatos) {
        String[] arrayDatos = cadenaDatos.split("\\s");
        ArrayList<String> listaDatos = new ArrayList<String>();
        for (int i = 0; i < arrayDatos.length; i++)
            if (arrayDatos[i].compareTo("") != 0)
                listaDatos.add(arrayDatos[i]);
        return listaDatos.toArray(new String[1]);
    }
    
    /**
     * Busca un nodo en el grafo a partir de las coordenadas recibidas
     * como parmetros
     * @param x coordenada X
     * @param y coordenada Y
     * @return una instancia de IdElemento que, en caso de que se encuentre un nodo,
     * indicar que el elemento encontrado es un nodo y contendr el identificador
     * del nodo. En caso de que no se encuentre un nodo, la instancia de IdElemento
     * indicar que el tipo de elemento es "vaco".
     */
    public IdElemento localizarNodo(int x, int y) {
        Iterator<Nodo> itNodos = nodos.iterator();
        int i = 0;
        while (itNodos.hasNext()) {
            Nodo n = itNodos.next();
            if (n.esNodoBuscado(x, y))
                return new IdElemento(IdElemento.NODO, i);
            i++;
        }
        return new IdElemento();
    }
    
    /**
     * Busca una arista en el grafo a partir de las coordenadas recibidas
     * como parmetros
     * @param x coordenada X
     * @param y coordenada Y
     * @return una instancia de IdElemento que, en caso de que se encuentre una arista,
     * indicar que el elemento encontrado es una arista y contendr el identificador
     * de la arista. En caso de que no se encuentre una arista, la instancia de IdElemento
     * indicar que el tipo de elemento es "vaco".
     */
    public IdElemento localizarArista(int x, int y) {
        Iterator<Arista> itAristas = aristas.iterator();
        int i = 0;
        while (itAristas.hasNext()) {
            Arista a = itAristas.next();
            if (a.esAristaBuscada(x, y))
                return new IdElemento(IdElemento.ARISTA, i);
            i++;
        }
        return new IdElemento();
    }
    
    /**
     * Busca un elemento en el grafo a partir de las coordenadas recibidas
     * como parmetros
     * @param x coordenada X
     * @param y coordenada Y
     * @return una instancia de IdElemento que, en caso de que se encuentre un elemento,
     * indicar de qu tipo es el elemento y contendr el identificador
     * del elemento. En caso de que no se encuentre un elemento, la instancia de IdElemento
     * indicar que el tipo de elemento es "vaco".
     */
    public IdElemento localizarElemento(int x, int y) {
        IdElemento id = localizarNodo(x, y);
        if (id.tipo == IdElemento.NODO)
            return id;
        else
            return localizarArista(x, y);
    }
    
    /**
     * Selecciona un nodo dado el identificador del mismo
     * @param idNodo identificador del nodo a seleccionar
     */
    public void seleccionarNodo(int idNodo) {
        Nodo nodo = nodos.get(idNodo);
        if (!nodo.estaSeleccionado()) {
            nodosSeleccionados.add(nodo);
            nodo.setSeleccionado(true);
        }
    }
    
    /**
     * Si existen en el grafo nodos seleccionados, se deseleccionan
     * @return true si existen nodos seleccionados y, por tanto, se deseleccionan;
     * false en caso contrario
     */
    public boolean deseleccionarNodosSeleccionados() {
        final int TAMANYO_INICIAL = nodosSeleccionados.size();
        if (TAMANYO_INICIAL == 0)
            return false;
        for (int i = TAMANYO_INICIAL - 1; i >= 0; i--) {
            nodosSeleccionados.get(i).setSeleccionado(false);
            nodosSeleccionados.remove(i);
        }
        return true;
    }
    
    /**
     * Selecciona una arista dado el identificador de la misma
     * @param idArista identificador de la arista a seleccionar
     */
    public void seleccionarArista(int idArista) {
        Arista arista = aristas.get(idArista);
        if (!arista.estaSeleccionada()) {
            aristasSeleccionadas.add(arista);
            arista.setSeleccionada(true);
        }
    }
    
    /**
     * Si existen en el grafo aristas seleccionadas, se deseleccionan
     * @return true si existen aristas seleccionadas y, por tanto, se deseleccionan;
     * false en caso contrario
     */
    public boolean deseleccionarAristasSeleccionadas() {
        final int TAMANYO_INICIAL = aristasSeleccionadas.size();
        if (TAMANYO_INICIAL == 0)
            return false;
        for (int i = TAMANYO_INICIAL - 1; i >= 0; i--) {
            aristasSeleccionadas.get(i).setSeleccionada(false);
            aristasSeleccionadas.remove(i);
        }
        return true;
    }
    
    /**
     * Comprueba si en el grafo hay exactamente un nodo seleccionado y, si es as,
     * localiza el nodo en cuestin
     * @return una instancia de IdElemento que, en caso de que se localice el nodo,
     * indicar que el elemento localizado es un nodo y contendr el identificador
     * del nodo. En caso de que no se encuentre el nodo, la instancia de IdElemento
     * indicar que el tipo de elemento es "vaco".
     */
    public IdElemento localizarNodoSeleccionado() {
        if (nodosSeleccionados.size() != 1)
            return new IdElemento();
        int i = indiceEnNodosDe(nodosSeleccionados.get(0));
        if (i >= 0)
            return new IdElemento(IdElemento.NODO, i);
        return new IdElemento();
    }
    
    /**
     * Comprueba si en el grafo hay exactamente una arista seleccionada y, si es as,
     * localiza la arista en cuestin
     * @return una instancia de IdElemento que, en caso de que se localice la arista,
     * indicar que el elemento localizado es una arista y contendr el identificador
     * de la arista. En caso de que no se encuentre la arista, la instancia de IdElemento
     * indicar que el tipo de elemento es "vaco".
     */
    public IdElemento localizarAristaSeleccionada() {
        if (aristasSeleccionadas.size() != 1)
            return new IdElemento();
        int i = indiceEnAristasDe(aristasSeleccionadas.get(0));
        if (i >= 0)
            return new IdElemento(IdElemento.ARISTA, i);
        return new IdElemento();
    }
    
    /**
     * Crea en el grafo un nodo con las coordenadas recibidas como parmetros
     * @param x coordenada X del nuevo nodo
     * @param y coordenada Y del nuevo nodo
     */
    public void crearNodo(int x, int y) {
        nodos.add(new Nodo(x, y));
    }
    
    /**
     * Crea en el grafo una arista dados los identificadores de los nodos origen y destino
     * de la misma y su 
     * @param idOrigen identificador del nodo origen
     * @param idDestino identificador del nodo destino
     * @return true en caso de exito; false en caso de error (que se producir si ya
     * existe una arista incidente con los dos nodos)
     */
    public boolean crearArista(int idOrigen, int idDestino) {
        deseleccionarNodosSeleccionados();
        
        /*
         * Se comprueba si existe ya una arista incidente con los dos nodos (y se devuelve
         * false en caso de que exista)
         */
        Arista arista;
        Nodo nodoOrigen = nodos.get(idOrigen);
        Nodo nodoDestino = nodos.get(idDestino);
        Iterator<Arista> itAristas = aristas.iterator();
        while (itAristas.hasNext()) {
            arista = itAristas.next();
            if (arista.incidenteCon(nodoOrigen, nodoDestino))
                return false;
        }
        
        // Se crea la arista (y se devuelve true)
        aristas.add(new Arista(nodoOrigen, nodoDestino, preferencias.siguientePeso()));
        return true;
    }
    
    /**
     * Cambia el peso de una arista
     * @param idArista identificador de la arista
     * @param nuevoPeso el nuevo peso de la arista
     */
    public void cambiarPesoArista(int idArista, int nuevoPeso) {
        aristas.get(idArista).cambiarPeso(nuevoPeso);
    }
    
    /**
     * Borra el nodo con el identificador recibido como parmetro sin comprobar si
     * est seleccionado (se da por supuesto que el nodo no est seleccionado)
     * @param idNodo identificador de un nodo
     */
    private void borrarNodoNoSeleccionado(int idNodo) {
        Nodo nodo = nodos.get(idNodo);
        
        // Se buscan las aristas incidentes con el nodo a borrar
        ArrayList<Integer> indices = new ArrayList<Integer>();
        Iterator<Arista> itAristas = aristas.iterator();
        int i = 0;
        while (itAristas.hasNext()) {
            Arista arista = itAristas.next();
            if (arista.incidenteCon(nodo))
                indices.add(new Integer(i));
            else
                i++;
        }
        
        // Se borran las aristas incidentes con el nodo a borrar
        Iterator<Integer> itIndices = indices.iterator();
        Arista arista;
        while (itIndices.hasNext()) {
            arista = aristas.remove(itIndices.next().intValue());
            arista.setSeleccionada(false);
        }
        final int TAMANYO_INICIAL = aristasSeleccionadas.size();
        for (i = TAMANYO_INICIAL - 1; i >= 0; i--)
            if (!aristasSeleccionadas.get(i).estaSeleccionada())
                aristasSeleccionadas.remove(i);
        
        // Se borra el nodo
        nodos.remove(idNodo);
    }
    
    /**
     * Si existe en el grafo nodos seleccionados, se borran
     * @return true si existen nodos seleccionados y, por lo tanto, se borran;
     * false en caso contrario
     */
    public boolean borrarNodosSeleccionados() {
        final int TAMANYO_INICIAL = nodosSeleccionados.size();
        if (TAMANYO_INICIAL == 0)
            return false;
        boolean borradosNodos = false;
        for (int i = TAMANYO_INICIAL - 1; i >= 0; i--) {
            int indiceEnNodos = indiceEnNodosDe(nodosSeleccionados.get(i));
            if (indiceEnNodos >= 0) {
                borrarNodoNoSeleccionado(indiceEnNodos);
                borradosNodos = true;
            }
            nodosSeleccionados.remove(i);
        }
        return borradosNodos;
    }
    
    /**
     * Si existen en el grafo aristas seleccionadas, se borran
     * @return true si existen aristas seleccionadas y, por lo tanto, se borran;
     * false en caso contrario
     */
    public boolean borrarAristasSeleccionadas() {
        final int TAMANYO_INICIAL = aristasSeleccionadas.size();
        if (TAMANYO_INICIAL == 0)
            return false;
        boolean borradasAristas = false;
        for (int i = TAMANYO_INICIAL - 1; i >= 0; i--) {
            int indiceEnAristas = indiceEnAristasDe(aristasSeleccionadas.get(i));
            if (indiceEnAristas >= 0) {
                aristas.remove(indiceEnAristas);
                borradasAristas = true;
            }
            aristasSeleccionadas.remove(i);
        }
        return borradasAristas;
    }
    
    /**
     * Actualiza la posicin de un nodo
     * @param idNodo identificador del nodo
     * @param nuevaX coordenada X de la nueva posicin
     * @param nuevaY coordenada Y de la nueva posicin
     */
    public void actualizarPosicinNodo(int idNodo, int nuevaX, int nuevaY) {
        nodos.get(idNodo).actualizarPosicin(nuevaX, nuevaY);
    }
    
    /**
     * Pinta este grafo
     * @param g contexto grfico para pintar el grafo
     */
    public void pintar(Graphics g) {
        
        // Se pintan los nodos
        Iterator<Nodo> itNodos = nodos.iterator();
        while (itNodos.hasNext())
            itNodos.next().pintar(g);
        
        // Se pintan las aristas
        Iterator<Arista> itAristas = aristas.iterator();
        while (itAristas.hasNext())
            itAristas.next().pintar(g);
        
        // Se pintan las etiquetas y/o los pesos de las aristas
        itAristas = aristas.iterator();
        while (itAristas.hasNext())
            itAristas.next().pintarTexto(g);
        
        // Se pintan las etiquetas de los nodos si stos estn etiquetados
        itNodos = nodos.iterator();
        while (itNodos.hasNext())
            itNodos.next().pintarEtiqueta(g);
    }
    
    /**
     * Convierte el grafo a una cadena de caracteres (a partir de la cual se podr
     * reconstruir el grafo posteriormente)
     * @return la cadena resultante de la conversin
     */
    public String cadenaDeGrafo() {
        StringWriter sw = new StringWriter();
        PrintWriter pw = new PrintWriter(sw);
        pw.println("fichero datos grafo");
        Iterator<Nodo> itNodos = nodos.iterator();
        while (itNodos.hasNext()) {
            Nodo n = itNodos.next();
            pw.print(" " + n.getX() + " " + n.getY());
        }
        pw.println();
        if (nodos.size() == 0) {
            pw.close();
            return sw.toString();
        }
        Iterator<Arista> itAristas = aristas.iterator();
        while (itAristas.hasNext()) {
            Arista a = itAristas.next();
            int idOrigen = indiceEnNodosDe(a.getNodoOrigen());
            int idDestino = indiceEnNodosDe(a.getNodoDestino());
            pw.print(" " + idOrigen + " " + idDestino + " " + a.getPeso());
        }
        pw.println();
        pw.close();
        return sw.toString();
    }
    
    /**
     * Indica si el grafo es conexo
     * @return true si el grafo es conexo; false en caso contrario.
     */
    public boolean esConexo() {
        int numNodos = nodos.size();
        if (numNodos == 0)
            return false;
        actualizarAdyacencias(numNodos);
        actualizarIdsNodosOrigenYDestino();
        int numConectados = 1;
        if (numConectados == numNodos)
            return true;
        boolean[] estaConectado = new boolean[numNodos];
        Arrays.fill(estaConectado, false);
        estaConectado[0] = true;
        LinkedList<Integer> nodosAVisitar = new LinkedList<Integer>();
        nodosAVisitar.add(new Integer(0));
        Integer entero;
        while ((entero = nodosAVisitar.poll()) != null) {
            int idNodoActual = entero.intValue();
            for (int i = 0; i < numNodos; i++) {
                if (adyacencias[idNodoActual][i] >= 0) {
                    if (!estaConectado[i]) {
                        numConectados++;
                        if (numConectados == numNodos)
                            return true;
                        estaConectado[i] = true;
                        nodosAVisitar.add(new Integer(i));
                    }
                }
            }
        }
        return false;
    }
    
    /**
     * Actualiza la matriz de adyacencias del grafo
     * @param numNodos nmero de nodos del grafo
     */
    private void actualizarAdyacencias(int numNodos) {
        adyacencias = new int[numNodos][numNodos];
        for (int i = 0; i < numNodos; i++)
            Arrays.fill(adyacencias[i], -1);
        int numAristas = aristas.size();
        for (int i = 0; i < numAristas; i++) {
            Arista a = aristas.get(i);
            int idOrigen = indiceEnNodosDe(a.getNodoOrigen());
            int idDestino = indiceEnNodosDe(a.getNodoDestino());
            adyacencias[idOrigen][idDestino] = i;
            adyacencias[idDestino][idOrigen] = i;
        }
    }
    
    /**
     * Actualiza los arrays de identificadores de nodos origen y destino de las aristas
     * del grafo
     */
    private void actualizarIdsNodosOrigenYDestino() {
        int numAristas = aristas.size();
        idsNodosOrigen = new int[numAristas];
        idsNodosDestino = new int[numAristas];
        for (int i = 0; i < numAristas; i++) {
            Arista a = aristas.get(i);
            idsNodosOrigen[i] = indiceEnNodosDe(a.getNodoOrigen());
            idsNodosDestino[i] = indiceEnNodosDe(a.getNodoDestino());
        }
    }
    
    /**
     * Comprueba si est seleccionado exactamente un nodo del grafo
     * @return true en caso positivo; false en caso contrario
     */
    public boolean seleccionadoUnNodo() {
        return nodosSeleccionados.size() == 1;
    }
    
    /**
     * Comprueba si est seleccionada exactamente una arista del grafo
     * @return true en caso positivo; false en caso contrario
     */
    public boolean seleccionadaUnaArista() {
        return aristasSeleccionadas.size() == 1;
    }
    
    /**
     * Comprueba si est seleccionada alguna arista del grafo
     * @return true en caso positivo; false en caso contrario
     */
    public boolean seleccionadasAristas() {
        return aristasSeleccionadas.size() > 0;
    }
    
    /**
     * Si existen en el grafo nodos marcados como pertenecientes a un rbol generador
     * construido o en construccin, se desmarcan
     * @return true si existen nodos marcados y, por tanto, se desmarcan;
     * false en caso contrario
     */
    public boolean desmarcarNodosMarcados() {
        final int TAMANYO_INICIAL = nodosDeArbol.size();
        if (TAMANYO_INICIAL == 0)
            return false;
        for (int i = TAMANYO_INICIAL - 1; i >= 0; i--) {
            nodosDeArbol.get(i).setPerteneceAArbol(false);
            nodosDeArbol.remove(i);
        }
        return true;
    }
    
    /**
     * Si existen en el grafo aristas marcadas como pertenecientes a un rbol generador
     * construido o en construccin, se desmarcan
     * @return true si existen aristas marcadas y, por tanto, se desmarcan;
     * false en caso contrario
     */
    public boolean desmarcarAristasMarcadas() {
        final int TAMANYO_INICIAL = aristasDeArbol.size();
        if (TAMANYO_INICIAL == 0)
            return false;
        for (int i = TAMANYO_INICIAL - 1; i >= 0; i--) {
            aristasDeArbol.get(i).setPerteneceAArbol(false);
            aristasDeArbol.remove(i);
        }
        return true;
    }
    
    /**
     * Si existen en el grafo nodos marcados como pertenecientes a un rbol auxiliar,
     * se desmarcan
     * @return true si existen nodos marcados y, por tanto, se desmarcan;
     * false en caso contrario
     */
    public boolean desmarcarNodosMarcadosArbolAux() {
        final int TAMANYO_INICIAL = nodosDeArbolAux.size();
        if (TAMANYO_INICIAL == 0)
            return false;
        for (int i = TAMANYO_INICIAL - 1; i >= 0; i--) {
            nodosDeArbolAux.get(i).setPerteneceAArbolAux(false);
            nodosDeArbolAux.remove(i);
        }
        return true;
    }
    
    /**
     * Si existen en el grafo aristas marcadas como pertenecientes a un rbol auxiliar,
     * se desmarcan
     * @return true si existen aristas marcadas y, por tanto, se desmarcan;
     * false en caso contrario
     */
    public boolean desmarcarAristasMarcadasArbolAux() {
        final int TAMANYO_INICIAL = aristasDeArbolAux.size();
        if (TAMANYO_INICIAL == 0)
            return false;
        for (int i = TAMANYO_INICIAL - 1; i >= 0; i--) {
            aristasDeArbolAux.get(i).setPerteneceAArbolAux(false);
            aristasDeArbolAux.remove(i);
        }
        return true;
    }
    
    /**
     * Limpia el grafo, es decir, deselecciona los nodos y aristas que estn seleccionados,
     * desmarca los nodos y aristas que estn marcados como pertenecientes a un rbol
     * (auxiliar o no auxiliar) y desetiqueta los nodos.
     */
    public void limpiar() {
        deseleccionarNodosSeleccionados();
        deseleccionarAristasSeleccionadas();
        desmarcarNodosMarcados();
        desmarcarAristasMarcadas();
        desmarcarNodosMarcadosArbolAux();
        desmarcarAristasMarcadasArbolAux();
        desetiquetarNodos();
        desetiquetarAristas();
    }
    
    /**
     * Devuelve la matriz de adyacencias del grafo
     * @return la matriz de adyacencias del grafo
     */
    public int[][] getAdyacencias() {
        return adyacencias;
    }
    
    /**
     * Devuelve el array de identificadores de nodos origen de las aristas del grafo
     * @return el array de identificadores de nodos origen de las aristas del grafo
     */
    public int[] getIdsNodosOrigen() {
        return idsNodosOrigen;
    }
    
    /**
     * Devuelve el array de identificadores de nodos destino de las aristas del grafo
     * @return el array de identificadores de nodos destino de las aristas del grafo
     */
    public int[] getIdsNodosDestino() {
        return idsNodosDestino;
    }
    
    /**
     * Marca un nodo como perteneciente a un rbol generador
     * @param idNodo el identificador del nodo a marcar
     */
    public void marcarNodo(int idNodo) {
        Nodo nodo = nodos.get(idNodo);
        if (!nodo.perteneceAArbol()) {
            nodosDeArbol.add(nodo);
            nodo.setPerteneceAArbol(true);
        }
    }
    
    /**
     * Marca una arista como perteneciente a un rbol generador
     * @param idArista el identificador de la arista a marcar
     */
    public void marcarArista(int idArista) {
        Arista arista = aristas.get(idArista);
        if (!arista.perteneceAArbol()) {
            aristasDeArbol.add(arista);
            arista.setPerteneceAArbol(true);
        }
    }
    
    /**
     * Desmarca una arista marcada como perteneciente a un rbol generador
     * @param idArista el identificador de la arista a desmarcar
     */
    public void desmarcarArista(int idArista) {
        Arista arista = aristas.get(idArista);
        if (arista.perteneceAArbol()) {
            int numAristas = aristasDeArbol.size();
            for (int i = 0; i < numAristas; i++) {
                if (aristasDeArbol.get(i) == arista) {
                    aristasDeArbol.remove(i);
                    break;
                }
            }
            arista.setPerteneceAArbol(false);
        }
    }
    
    /**
     * Marca un nodo como perteneciente a un rbol auxiliar
     * @param idNodo el identificador del nodo a marcar
     */
    public void marcarNodoArbolAux(int idNodo) {
        Nodo nodo = nodos.get(idNodo);
        if (!nodo.perteneceAArbolAux()) {
            nodosDeArbolAux.add(nodo);
            nodo.setPerteneceAArbolAux(true);
        }
    }
    
    /**
     * Marca una arista como perteneciente a un rbol auxiliar
     * @param idArista el identificador de la arista a marcar
     */
    public void marcarAristaArbolAux(int idArista) {
        Arista arista = aristas.get(idArista);
        if (!arista.perteneceAArbolAux()) {
            aristasDeArbolAux.add(arista);
            arista.setPerteneceAArbolAux(true);
        }
    }
    
    /**
     * Cambia el estado de un nodo del grafo: si no est seleccionado, se selecciona, y
     * si est seleccionado, se deselecciona
     * @param idNodo identificador del nodo a seleccionar/deseleccionar
     */
    public void cambiarEstadoSeleccionadoNodo(int idNodo) {
        Nodo nodo = nodos.get(idNodo);
        if (!nodo.estaSeleccionado()) {
            nodosSeleccionados.add(nodo);
            nodo.setSeleccionado(true);
        }
        else {
            int numNodosSel = nodosSeleccionados.size();
            for (int i = 0; i < numNodosSel; i++) {
                if (nodosSeleccionados.get(i) == nodo) {
                    nodo.setSeleccionado(false);
                    nodosSeleccionados.remove(i);
                    return;
                }
            }
        }
    }
    
    /**
     * Cambia el estado de una arista del grafo: si no est seleccionada, se selecciona, y
     * si est seleccionada, se deselecciona
     * @param idArista identificador de la arista a seleccionar/deseleccionar
     */
    public void cambiarEstadoSeleccionadaArista(int idArista) {
        Arista arista = aristas.get(idArista);
        if (!arista.estaSeleccionada()) {
            aristasSeleccionadas.add(arista);
            arista.setSeleccionada(true);
        }
        else {
            int numAristasSel = aristasSeleccionadas.size();
            for (int i = 0; i < numAristasSel; i++) {
                if (aristasSeleccionadas.get(i) == arista) {
                    arista.setSeleccionada(false);
                    aristasSeleccionadas.remove(i);
                    return;
                }
            }
        }
    }
    
    /**
     * Localiza todas las aristas del grafo que estn seleccionadas
     * @return un array de identificadores de las aristas seleccionadas si hay alguna
     * arista seleccionada; null en caso contrario
     */
    public int[] localizarAristasSeleccionadas() {
        int numAristas = aristas.size();
        if (numAristas == 0)
            return null;
        int[] idsAristasSel = new int[aristasSeleccionadas.size()];
        int iAristasSel = 0;
        for (int iAristas = 0; iAristas < numAristas; iAristas++) {
            if (aristas.get(iAristas).estaSeleccionada()) {
                idsAristasSel[iAristasSel] = iAristas;
                iAristasSel++;
                if (iAristasSel == idsAristasSel.length)
                    return idsAristasSel;
            }
        }
        return idsAristasSel;
    }
    
    /**
     * Devuelve el nmero de nodos del grafo
     * @return el nmero de nodos del grafo
     */
    public int numNodos() {
        return nodos.size();
    }
    
    /**
     * Devuelve el nmero de aristas del grafo
     * @return el nmero de aristas del grafo
     */
    public int numAristas() {
        return aristas.size();
    }
    
    /**
     * Devuelve la suma de los pesos de las aristas del grafo
     * @return la suma de los pesos de las aristas del grafo
     */
    public long sumaPesosGrafo() {
        long sumaPesos = 0;
        int numAristas = aristas.size();
        for (int i = 0; i < numAristas; i++) {
            sumaPesos += aristas.get(i).getPeso();
        }
        return sumaPesos;
    }
    
    /**
     * Etiqueta los nodos del grafo
     * @param etiquetasNodos array de etiquetas de los nodos
     */
    public void etiquetarNodos(String[] etiquetasNodos) {
        int numNodos = nodos.size();
        for (int i = 0; i < numNodos; i++)
            nodos.get(i).etiquetar(etiquetasNodos[i]);
    }
    
    /**
     * Desetiqueta los nodos del grafo
     */
    public void desetiquetarNodos() {
        int numNodos = nodos.size();
        for (int i = 0; i < numNodos; i++)
            nodos.get(i).desetiquetar();
    }
    
    /**
     * Etiqueta las aristas del grafo
     * @param etiquetasAristas array de etiquetas de las aristas
     */
    public void etiquetarAristas(String[] etiquetasAristas) {
        int numAristas = aristas.size();
        for (int i = 0; i < numAristas; i++)
            aristas.get(i).etiquetar(etiquetasAristas[i]);
    }
    
    /**
     * Desetiqueta los nodos del grafo
     */
    public void desetiquetarAristas() {
        int numAristas = aristas.size();
        for (int i = 0; i < numAristas; i++)
            aristas.get(i).desetiquetar();
    }
    
}
