package interfaz;

import javax.swing.*;

import java.awt.*;
import java.util.*;

import estructuras.*;

/**
 * Clase que implementa el rea en el que se permitir dibujar un grafo
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class AreaDibujoGrafo extends JPanel {

    private static final long serialVersionUID = 1L;
    private static final Color COLOR_FONDO = Color.white;
    
    private Grafo grafo;
    private boolean grafoEditable = true;
    private boolean etiquetadosNodos = false;
    private boolean etiquetadasAristas = false;
    private Preferencias preferencias;
    
    /**
     * Constructor
     * @param preferencias preferencias del usuario
     */
    public AreaDibujoGrafo(Preferencias preferencias) {
        super();
        grafo = new Grafo(preferencias);
        this.preferencias = preferencias;
        setPreferredSize(new Dimension(4000,3000));
        setBackground(COLOR_FONDO);
        setFocusable(true);
    }
    
    /**
     * Redefinicin del mtodo paintComponent(Graphics g) de la clase JPanel
     * (para que se pinte el grafo)
     */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        grafo.pintar(g);
    }
    
    /**
     * Indica si el grafo es editable
     * @return true si el grafo es editable; false en caso contrario.
     */
    public boolean grafoEsEditable() {
        return grafoEditable;
    }
    
    /**
     * Busca un nodo en el rea de dibujo a partir de las coordenadas recibidas
     * como parmetros
     * @param x coordenada X
     * @param y coordenada Y
     * @return una instancia de IdElemento que, en caso de que se encuentre un nodo,
     * indicar que el elemento encontrado es un nodo y contendr el identificador
     * del nodo. En caso de que no se encuentre un nodo, la instancia de IdElemento
     * indicar que el tipo de elemento es "vaco".
     */
    public IdElemento localizarNodo(int x, int y) {
        return grafo.localizarNodo(x, y);
    }
    
    /**
     * Busca un elemento en el rea de dibujo a partir de las coordenadas recibidas
     * como parmetros
     * @param x coordenada X
     * @param y coordenada Y
     * @return una instancia de IdElemento que, en caso de que se encuentre un elemento,
     * indicar de qu tipo es el elemento y contendr el identificador
     * del elemento. En caso de que no se encuentre un elemento, la instancia de IdElemento
     * indicar que el tipo de elemento es "vaco".
     */
    public IdElemento localizarElemento(int x, int y) {
        return grafo.localizarElemento(x, y);
    }
    
    /**
     * Selecciona un nodo dado el identificador del mismo
     * @param idNodo identificador del nodo a seleccionar
     */
    public void seleccionarNodo(int idNodo) {
        grafo.seleccionarNodo(idNodo);
        repaint();
    }
    
    /**
     * Si existen en el rea de dibujo nodos seleccionados, se deseleccionan
     *
     */
    public void deseleccionarNodosSeleccionados() {
        if (grafo.deseleccionarNodosSeleccionados())
            repaint();
    }
    
    /**
     * Selecciona una arista dado el identificador de la misma
     * @param idArista identificador de la arista a seleccionar
     */
    public void seleccionarArista(int idArista) {
        grafo.seleccionarArista(idArista);
        repaint();
    }
    
    /**
     * Si existen en el rea de dibujo aristas seleccionadas, se deseleccionan
     *
     */
    public void deseleccionarAristasSeleccionadas() {
        if (grafo.deseleccionarAristasSeleccionadas())
            repaint();
    }
    
    /**
     * Si existen en el rea de dibujo elementos seleccionados, se deseleccionan
     *
     */
    public void deseleccionarElementosSeleccionados() {
        boolean deseleccionadosNodos = grafo.deseleccionarNodosSeleccionados();
        boolean deseleccionadasAristas = grafo.deseleccionarAristasSeleccionadas();
        if (deseleccionadosNodos || deseleccionadasAristas)
            repaint();
    }
    
    /**
     * Comprueba si en el grafo hay exactamente un nodo seleccionado y, si es as,
     * localiza el nodo en cuestin
     * @return una instancia de IdElemento que, en caso de que se encuentre el nodo,
     * indicar que el elemento encontrado es un nodo y contendr el identificador
     * del nodo. En caso de que no se encuentre el nodo, la instancia de IdElemento
     * indicar que el tipo de elemento es "vaco".
     */
    public IdElemento localizarNodoSeleccionado() {
        return grafo.localizarNodoSeleccionado();
    }
    
    /**
     * Comprueba si en el grafo hay exactamente una arista seleccionada y, si es as,
     * localiza la arista en cuestin
     * @return una instancia de IdElemento que, en caso de que se encuentre la arista,
     * indicar que el elemento encontrado es una arista y contendr el identificador
     * de la arista. En caso de que no se encuentre la arista, la instancia de IdElemento
     * indicar que el tipo de elemento es "vaco".
     */
    public IdElemento localizarAristaSeleccionada() {
        return grafo.localizarAristaSeleccionada();
    }
    
    /**
     * Crea en el rea de dibujo un nodo con las coordenadas recibidas como parmetros
     * @param x coordenada X del nuevo nodo
     * @param y coordenada Y del nuevo nodo
     */
    public void crearNodo(int x, int y) {
        grafo.crearNodo(x, y);
        repaint();
    }
    
    /**
     * Crea en el rea de dibujo una arista dados los identificadores de los nodos
     * origen y destino de la misma
     * @param idOrigen identificador del nodo origen
     * @param idDestino identificador del nodo destino
     */
    public void crearArista(int idOrigen, int idDestino) {
        if (grafo.crearArista(idOrigen, idDestino))
            repaint();
        else {
            Object[] opciones = {"OK"};
            JOptionPane.showOptionDialog(null, "The edge already exists, multiple edges " +
                    "are not allowed", "Error",
                    JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
                    null, opciones, null);
        }
    }
    
    /**
     * Permite al usuario cambiar el peso de una arista existente en el rea de dibujo
     * @param idArista identificador de la arista
     */
    public void permitirCambiarPesoArista(int idArista) {
        
        //Se prepara un panel que incluya la etiqueta y el campo de texto para la ventana
        //en la que se pedir al usuario el nuevo peso para una arista
        JLabel etiqueta = new JLabel("Enter the new edge weight");
        JTextField campoTexto = new JTextField(14); //Campo de texto de 14 columnas
        JPanel panel = new JPanel();
        panel.setLayout(new FlowLayout(FlowLayout.LEFT));
        panel.setPreferredSize(new Dimension(200, 45));
        panel.add(etiqueta);
        panel.add(campoTexto);
        
        Object[] opcionesEntrada = {"OK", "Cancel"};
        if (JOptionPane.showOptionDialog(null, panel, "Change weight",
                JOptionPane.DEFAULT_OPTION, JOptionPane.QUESTION_MESSAGE, null, opcionesEntrada,
                null) == 0) {
            String cadenaPesoArista = campoTexto.getText();
            if (cadenaPesoArista != null) {
                Object[] opcionesError = {"OK"};
                if (cadenaPesoArista.length() > 0) {
                    try {
                        int pesoArista = Integer.parseInt(cadenaPesoArista);
                        if ((pesoArista <= 0) || (pesoArista == Integer.MAX_VALUE))
                            JOptionPane.showOptionDialog(null, "Invalid weight", "Error",
                                    JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
                                    null, opcionesError, null);
                        else {
                            grafo.cambiarPesoArista(idArista, pesoArista);
                            repaint();
                        }
                    } catch (Exception e) {
                        JOptionPane.showOptionDialog(null, "Invalid weight", "Error",
                                JOptionPane.DEFAULT_OPTION, JOptionPane.ERROR_MESSAGE,
                                null, opcionesError, null);
                    }
                }
            }
        }
    }
    
    /**
     * Si hay una arista seleccionada, permite al usuario cambiar el peso de la misma
     */
    public void permitirCambiarPesoAristaSeleccionada() {
        IdElemento idElemento = localizarAristaSeleccionada();
        if (idElemento.tipo == IdElemento.ARISTA)
            permitirCambiarPesoArista(idElemento.id);
    }
    
    /**
     * Si hay un elemento seleccionado en el rea de dibujo, se borra dicho elemento
     *
     */
    public void borrarSeleccionados() {
        boolean borradosNodos = grafo.borrarNodosSeleccionados();
        boolean borradasAristas = grafo.borrarAristasSeleccionadas();
        if (borradosNodos || borradasAristas)
            repaint();
    }
    
    /**
     * Mueve un nodo a una nueva posicin
     * @param idNodo identificador del nodo
     * @param nuevaX coordenada X de la nueva posicin
     * @param nuevaY coordenada Y de la nueva posicin
     */
    public void moverNodo(int idNodo, int nuevaX, int nuevaY) {
        grafo.actualizarPosicinNodo(idNodo, nuevaX, nuevaY);
        repaint();
    }
    
    /**
     * Convierte el grafo a una cadena de caracteres (a partir de la cual se podr
     * reconstruir el grafo posteriormente)
     * @return la cadena resultante de la conversin
     */
    public String cadenaDeGrafo() {
        return grafo.cadenaDeGrafo();
    }
    
    /**
     * Reconstruye el grafo a partir de una cadena de caracteres
     * @param cadena cadena a partir de la cual se debe reconstruir el grafo
     */
    public void reconstruirGrafo(String cadena) {
        grafo = new Grafo(cadena, preferencias);
        repaint();
    }
    
    /**
     * Limpia el area de dibujo
     */
    public void limpiar() {
        grafo = new Grafo(preferencias);
        repaint();
    }
    
    /**
     * Indica si el grafo es conexo
     * @return true si el grafo es conexo; false en caso contrario.
     */
    public boolean grafoEsConexo() {
        return grafo.esConexo();
    }
    
    /**
     * Establece si el grafo es editable
     * @param editable indica si el grafo pasa a ser editable
     */
    public void setGrafoEditable(boolean editable) {
        grafoEditable = editable;
        limpiarGrafo();
    }
    
    /**
     * Comprueba si est seleccionado exactamente un nodo del grafo
     * @return true en caso positivo; false en caso contrario
     */
    public boolean seleccionadoUnNodo() {
        return grafo.seleccionadoUnNodo();
    }
    
    /**
     * Comprueba si est seleccionada exactamente una arista del grafo
     * @return true en caso positivo; false en caso contrario
     */
    public boolean seleccionadaUnaArista() {
        return grafo.seleccionadaUnaArista();
    }
    
    /**
     * Comprueba si est seleccionada alguna arista del grafo
     * @return true en caso positivo; false en caso contrario
     */
    public boolean seleccionadasAristas() {
        return grafo.seleccionadasAristas();
    }
    
    /**
     * Limpia el grafo, es decir, deselecciona los nodos y aristas que estn seleccionados,
     * desmarca los nodos y aristas que estn marcados como pertenecientes a un rbol
     * generador construido o en construccin y desetiqueta los nodos.
     */
    public void limpiarGrafo() {
        grafo.limpiar();
        etiquetadosNodos = false;
        etiquetadasAristas = false;
        repaint();
    }
    
    /**
     * Devuelve la matriz de adyacencias del grafo
     * @return la matriz de adyacencias del grafo
     */
    public int[][] getAdyacencias() {
        return grafo.getAdyacencias();
    }
    
    /**
     * Devuelve el array de identificadores de nodos origen de las aristas del grafo
     * @return el array de identificadores de nodos origen de las aristas del grafo
     */
    public int[] getIdsNodosOrigen() {
        return grafo.getIdsNodosOrigen();
    }
    
    /**
     * Devuelve el array de identificadores de nodos destino de las aristas del grafo
     * @return el array de identificadores de nodos destino de las aristas del grafo
     */
    public int[] getIdsNodosDestino() {
        return grafo.getIdsNodosDestino();
    }
    
    /**
     * Devuelve la lista de nodos del grafo
     * @return la lista de nodos del grafo
     */
    public ArrayList<Nodo> getNodos() {
        return grafo.getNodos();
    }
    
    /**
     * Devuelve la lista de aristas del grafo
     * @return la lista de aristas del grafo
     */
    public ArrayList<Arista> getAristas() {
        return grafo.getAristas();
    }
    
    /**
     * Marca un nodo como perteneciente a un rbol generador
     * @param idNodo el identificador del nodo a marcar
     */
    public void marcarNodo(int idNodo) {
        grafo.marcarNodo(idNodo);
        repaint();
    }
    
    /**
     * Marca una arista como perteneciente a un rbol generador
     * @param idArista el identificador de la arista a marcar
     */
    public void marcarArista(int idArista) {
        grafo.marcarArista(idArista);
        repaint();
    }
    
    /**
     * Desmarca una arista marcada como perteneciente a un rbol generador
     * @param idArista el identificador de la arista a desmarcar
     */
    public void desmarcarArista(int idArista) {
        grafo.desmarcarArista(idArista);
        repaint();
    }
    
    /**
     * Marca un nodo como perteneciente a un rbol auxiliar
     * @param idNodo el identificador del nodo a marcar
     */
    public void marcarNodoArbolAux(int idNodo) {
        grafo.marcarNodoArbolAux(idNodo);
        repaint();
    }
    
    /**
     * Marca una arista como perteneciente a un rbol auxiliar
     * @param idArista el identificador de la arista a marcar
     */
    public void marcarAristaArbolAux(int idArista) {
        grafo.marcarAristaArbolAux(idArista);
        repaint();
    }
    
    /**
     * Cambia el estado de un nodo del grafo: si no est seleccionado, se selecciona, y
     * si est seleccionado, se deselecciona
     * @param idNodo identificador del nodo a seleccionar/deseleccionar
     */
    public void cambiarEstadoSeleccionadoNodo(int idNodo) {
        grafo.cambiarEstadoSeleccionadoNodo(idNodo);
        repaint();
    }
    
    /**
     * Cambia el estado de una arista del grafo: si no est seleccionada, se selecciona, y
     * si est seleccionada, se deselecciona
     * @param idArista identificador de la arista a seleccionar/deseleccionar
     */
    public void cambiarEstadoSeleccionadaArista(int idArista) {
        grafo.cambiarEstadoSeleccionadaArista(idArista);
        repaint();
    }
    
    /**
     * Localiza todas las aristas del grafo que estn seleccionadas
     * @return un array de identificadores de las aristas seleccionadas si hay alguna
     * arista seleccionada; null en caso contrario
     */
    public int[] localizarAristasSeleccionadas() {
        return grafo.localizarAristasSeleccionadas();
    }
    
    /**
     * Devuelve el nmero de nodos del grafo
     * @return el nmero de nodos del grafo
     */
    public int numNodosGrafo() {
        return grafo.numNodos();
    }
    
    /**
     * Devuelve el nmero de aristas del grafo
     * @return el nmero de aristas del grafo
     */
    public int numAristasGrafo() {
        return grafo.numAristas();
    }
    
    /**
     * Devuelve la suma de los pesos de las aristas del grafo
     * @return la suma de los pesos de las aristas del grafo
     */
    public long sumaPesosGrafo() {
        return grafo.sumaPesosGrafo();
    }
    
    /**
     * Etiqueta los nodos del grafo
     * @param etiquetasNodos array de etiquetas de los nodos
     */
    public void etiquetarNodos(String[] etiquetasNodos) {
        if (!etiquetadosNodos) {
            grafo.etiquetarNodos(etiquetasNodos);
            repaint();
            etiquetadosNodos = true;
        }
    }
    
    /**
     * Desetiqueta los nodos del grafo
     */
    public void desetiquetarNodos() {
        if (etiquetadosNodos) {
            grafo.desetiquetarNodos();
            repaint();
            etiquetadosNodos = false;
        }
    }
    
    /**
     * Etiqueta las aristas del grafo
     * @param etiquetasAristas array de etiquetas de las aristas
     */
    public void etiquetarAristas(String[] etiquetasAristas) {
        if (!etiquetadasAristas) {
            grafo.etiquetarAristas(etiquetasAristas);
            repaint();
            etiquetadasAristas = true;
        }
    }
    
    /**
     * Desetiqueta las aristas del grafo
     */
    public void desetiquetarAristas() {
        if (etiquetadasAristas) {
            grafo.desetiquetarAristas();
            repaint();
            etiquetadasAristas = false;
        }
    }
    
}
