package interfaz;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JPanel;

/**
 * Clase que implementa el panel pequeo en el que se representar un grafo a escala
 * 
 * @author Daniel Guijarro Enriquez
 * @version 1.0
 */
public class MinipanelGrafo extends JPanel {

    private static final long serialVersionUID = 1L;
    
    private String cadenaDeGrafo;
    private Grafo grafo;
    
    /**
     * Constructor
     * @param cadenaDeGrafo cadena de caracteres que representa un grafo
     */
    public MinipanelGrafo(String cadenaDeGrafo) {
        this.cadenaDeGrafo = cadenaDeGrafo;
        grafo = new Grafo(cadenaDeGrafo);
        setPreferredSize(new Dimension(240,180));
        setBackground(Color.white);
    }
    
    /**
     * Redefinicin del mtodo paintComponent(Graphics g) de la clase JPanel
     * (para que se pinte el grafo)
     */
    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        g.setColor(Color.blue);
        grafo.pintar(g);
    }
    
    public String cadenaDeGrafo() {
        return cadenaDeGrafo;
    }
    
    /**
     * Clase de objetos que representan grafos mostrados en el minipanel
     */
    class Grafo {
        
        private ArrayList<Nodo> nodos = new ArrayList<Nodo>();
        private ArrayList<Arista> aristas = new ArrayList<Arista>();
        
        /**
         * Constructor
         * @param cadena cadena de caracteres que representa un grafo
         */
        public Grafo(String cadena) {
            BufferedReader flujo = new BufferedReader(new StringReader(cadena));
            try {
                flujo.readLine();
                String linea = flujo.readLine();
                if (linea.compareTo("") == 0) {
                    flujo.close();
                    return;
                }
                String[] datosNodos = aArray(linea);
                for (int i = 1; i < datosNodos.length; i = i + 2)
                    crearNodo(Integer.parseInt(datosNodos[i - 1]) / 3,
                            Integer.parseInt(datosNodos[i]) / 3);
                linea = flujo.readLine();
                if (linea.compareTo("") == 0) {
                    flujo.close();
                    return;
                }
                String[] datosAristas = aArray(linea);
                for (int i = 2; i < datosAristas.length; i = i + 3) {
                    Nodo nodoOrigen = nodos.get(Integer.parseInt(datosAristas[i - 2]));
                    Nodo nodoDestino = nodos.get(Integer.parseInt(datosAristas[i - 1]));
                    aristas.add(new Arista(nodoOrigen, nodoDestino));
                }
                flujo.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        
        /**
         * Pinta este grafo
         * @param g contexto grfico para pintar el grafo
         */
        public void pintar(Graphics g) {
            
            // Se pintan los nodos
            Iterator<Nodo> itNodos = nodos.iterator();
            while (itNodos.hasNext())
                itNodos.next().pintar(g);
            
            // Se pintan las aristas
            Iterator<Arista> itAristas = aristas.iterator();
            while (itAristas.hasNext())
                itAristas.next().pintar(g);
        }
        
        /**
         * Convierte una cadena de datos para reconstruir un grafo a un array de subcadenas
         * de la cadena en cuestin, donde cada subcadena representa un dato.
         * @param cadenaDatos la cadena a convertir
         * @return el array resultante
         */
        private String[] aArray(String cadenaDatos) {
            String[] arrayDatos = cadenaDatos.split("\\s");
            ArrayList<String> listaDatos = new ArrayList<String>();
            for (int i = 0; i < arrayDatos.length; i++)
                if (arrayDatos[i].compareTo("") != 0)
                    listaDatos.add(arrayDatos[i]);
            return listaDatos.toArray(new String[1]);
        }
        
        /**
         * Crea en el grafo un nodo con las coordenadas recibidas como parmetros
         * @param x coordenada X del nuevo nodo
         * @param y coordenada Y del nuevo nodo
         */
        private void crearNodo(int x, int y) {
            nodos.add(new Nodo(x, y));
        }
        
    }
    
    /**
     * Clase de objetos que representan nodos mostrados en el minipanel
     */
    class Nodo {
        
        private static final int RADIO = 2;
        private int x;
        private int y;
        
        /**
         * Constructor
         * @param coorX coordenada X del nodo a crear
         * @param coorY coordenada Y del nodo a crear
         */
        public Nodo(int coorX, int coorY) {
            x = coorX;
            y = coorY;
        }
        
        /**
         * Devuelve la coordenada X del nodo
         * @return la coordenada X del nodo
         */
        public int getX() {
            return x;
        }
        
        /**
         * Devuelve la coordenada Y del nodo
         * @return la coordenada Y del nodo
         */
        public int getY() {
            return y;
        }
        
        /**
         * Pinta este nodo
         * @param g contexto grfico para pintar el nodo
         */
        public void pintar(Graphics g) {
            g.fillOval(x - RADIO, y - RADIO, 2 * RADIO, 2 * RADIO);
        }
        
    }
    
    /**
     * Clase que representa el grafo mostrado en el minipanel
     */
    class Arista {
        
        private Nodo nodoOrigen;
        private Nodo nodoDestino;
        
        /**
         * Constructor
         * @param origen nodo origen de la arista
         * @param destino nodo destino de la arista
         */
        public Arista(Nodo origen, Nodo destino) {
            nodoOrigen = origen;
            nodoDestino = destino;
        }
        
        /**
         * Pinta esta arista
         * @param g contexto grfico para pintar la arista
         */
        public void pintar(Graphics g) {
            int xOrigen = nodoOrigen.getX();
            int yOrigen = nodoOrigen.getY();
            int xDestino = nodoDestino.getX();
            int yDestino = nodoDestino.getY();
            g.drawLine(xOrigen, yOrigen, xDestino, yDestino);
        }
        
    }
    
}
